#define  UN_EXTERN


#include "ne_headers.h"

int
main(void)
{
	// variable declarations
	int a,j,k,n_index;
	double LogNormFactor = 0.0;  // the factor to add to the log probs ratio to make it not so absurdly small
	double LocusAves[gMAX_LOC];  // Monte Carlo averages for each locus
	double LocusVars[gMAX_LOC];  // Monte Carlo variances for each locus
	double *LocusSquares;  // Used in the calculation of Monte Carlo variance
	double theOldLocAv; // temporarially keep track of the old mean for computing the variance
	int boing = 0;
	GeneInfo *theGIP;
	LatentInfo *theLIP;  
	LogProb *theLP;  
	int n_zero;
	double TotalLike=0.0,TotVar=0.0,TVTerm1,TVTerm2;
	FILE *likefile;
	FILE *outf;
	FILE *seedfile;
	FILE *LocByLocFile;
	int seed1,seed2;
	int num_reps;
	int **logLocHist;
	int *logLocHistLow;
	double tempLoc;
	double CIup, CIdown;
	int sizeOfHist = 200; // number of entries in the histogram for each locus
	long seed_test1,seed_test2;  // for checking to make sure that the same number of calls to the base r.v. generator
	// have been made for different values of n_zero.  (This is part of the "
							  // "coupling-savvy" part of things).
	int num_n_zeroes;  // the number of different n_zeroes to evaluate at.
	char scrap_char[300];
	time_t ATimeStruct;
	int AlleleGens;  // to count the number of alleles x generations in the dataset
						 // this is used to determine how frequently to write dots out
						 // to the screen on the Mac version. 
	int ReportIteration;  // how often to write dots to the screen 



	// PRINT A LITTLE INTRODUCTION TO THE USER.
	printf("\n\nWelcome to MCLEEPS, Version 1.0.");
	printf("\n\nThis program uses an importance sampling method for computing a likelihood \n");
	printf("Curve for N_e from data on temporal changes in allele frequency");
	printf("\n\nThis program was written by\n\nEric Anderson (eriq@cqs.washington.edu) and");
	printf("\n\nEllen Williamson (ellenw@socrates.berkeley.edu)");
	printf("\n\nPlease send email to Eric with any questions or bug reports");
	printf("\n\nYour data should be in a file with the name of your choice.  The required");
	printf("\ndata format is specified in the documentation.  An example is provided in the file ");
	printf("named \"testfile.txt\" which is distributed with the program");	
		
	printf("\n\nPlease type the name of the input file\n->");
	scanf("%s",gINFILE);
	
	
	// instantiating some objects  
	theGIP = new GeneInfo; // reads the info in the "infile"
		
	// Deal with seeding the random number generator
	printf("\nYou should have two integers in a file called RandomSeedsIn.txt to seed the\nrandom number generator.");
	printf("\n\nIf you stop and restart a run, you should use the same random number seeds;");
	printf("\nthis program obtains variance reductions by correlated sampling from using the");
	printf("\nsame block of random numbers for evaluations at different N_e's");

	// Get the random seeds
	seedfile = fopen("RandomSeedsIn.txt","r");
	if(seedfile==NULL) {
		printf("\n\nWhoa! Unable to find file RandomSeedsIn.txt");
		printf("\nEnter two positve integers less than 10,000,000, and those will be used");
		printf("\nand stored in a file called RandomSeedsIn.txt");
		printf("\n\nEnter two positive integers (NO decimal points!!)\n->");
		scanf("%d %d", &seed1,&seed2);
		seedfile = fopen("RandomSeedsIn.txt","w");
		fprintf(seedfile,"%d   %d",seed1,seed2);

	}
	else {
		fscanf(seedfile,"%d%d", &seed1,&seed2);
	}

	fclose(seedfile);
	
	// seed the random number generator
	setall(seed1,seed2);


	// setting the global variables to their proper values
	num_gen = theGIP->T + 1;  // so, things will cycle from =0 to <num_gen
	num_loc = theGIP->numLoci; 
	num_reps = theGIP->numReps;
	num_n_zeroes = 1 + ((theGIP->nHigh - theGIP->nLow)/theGIP->nIncr);
	

	logLocHistLow = ivector(0,num_loc-1);
	logLocHist = imatrix(0,num_loc-1,0,sizeOfHist-1);

	// open and close the histogram file in write mode so that it deletes any old stuff in it.
	outf = fopen("aa_LocusHistogramsOut.txt","w");
	fclose(outf);

	likefile = fopen("aa_LogLikelihoodsOut.txt","w");

	
	echo_data(theGIP, (int)seed1,(int)seed2);

	// count the number of alleles
	AlleleGens = 0;
	for(j=0;j<num_loc;j++)  {
		AlleleGens += theGIP->sample[0].locus[j].numAlleles - 1;
	}
	AlleleGens *= num_gen;
	ReportIteration = 600 / AlleleGens + 1;   // value 600 determined empirically on my laptop
	
	printf("\n\nData and settings have been read in and echoed out to the file");
	printf("\naa_SettingsAndDataOut.txt.\n");
	printf("\nAt this juncture you should check that file to make sure that");
	printf("\nall the setting and data were read in correctly.\n");
	printf("\nEnter a character to continue.  i.e. type 'h' and RETURN\n->");
	scanf("%s",scrap_char);
	
	

	
	// print the seeds to the outfile:
	fprintf(likefile,"Random number generator seeded with %d and %d",seed1,seed2);
//	printf("\n\nRandom number generator seeded with %d and %d",seed1,seed2);
	//Now put the row titles down:
	fprintf(likefile,"\nNe     MC_Log_Likelihood\t"); 
	fprintf(likefile,"Lower_CI\tUpper_CI\t ");
	
	// print out the headers on the Locus by Locus LogL file
	LocByLocFile = fopen("aa_LocByLocLogLOut.txt","w");
	fprintf(LocByLocFile,"Ne");
	for(j=0;j<num_loc;j++)  {
		fprintf(LocByLocFile,"\tLoc%d_LogL\tLoc%d_CI_low\tLoc%d_CI_hi",
						j+1,j+1,j+1);
	}
	fclose(LocByLocFile);
	  
	// Sort the alleles at each locus into an order that should be advantageous for 
	// doing the computations
	theGIP->SortAlleles();

	// allocate memory to a place to store relevant logs of probabilities
	theLP = new LogProb(theGIP->numLoci);

	// allocate memory to the latent data (X), and relevant quantities
	theLIP = new LatentInfo(theGIP);

	// Set the 'NotLost' field for each time period and allele.  This corresponds to 
	// setting the variable \delta in the Genetics article.  
	theLIP->RefreshNotLost(theGIP);

	// allocate to a variable that is important for computing the variance.
	LocusSquares = dvector(0,num_loc-1);

	n_zero = theGIP->nLow*2;
	
	// just before entering this loop.  Note the TIME.
	gStartTime = time(NULL);
	
	// cycle over all the values of Ne you wish to calculate the likelihood for
	for (n_index = 0; n_index < num_n_zeroes; n_index++) {
		n_zero = theGIP->nLow*2 + theGIP->nIncr*2*n_index;

		// initializing LocusAves and Locus Vars to zeroes
		for(j=0;j<num_loc;j++) {
		LocusAves[j] = 0.0;
		LocusVars[j] = 0.0;
		LocusSquares[j] = 0.0;
			for (k = 0; k < sizeOfHist; k++)
				logLocHist[j][k] = 0;
		}

		// reset to the original random number seed each time:
		setall(seed1,seed2);
		// modifying the following line so it says nothing about rng seeds.
		//printf("\n\nStarting reps for Diploid Ne = %d.  Seed1 = %d, Seed2 = %d\n",n_zero/2,seed1,seed2);

//		printf("\n\nStarting reps for Diploid Ne = %d.",n_zero/2);
//		printf("   Number of Reps = %d\n",num_reps);


		// HERE WE BEGIN THE LOOP THAT DOES ALL THE MONTE CARLO REPLICATES
		for(a=0;a<num_reps;a++) {
#ifdef MAC_CODE_WARRIOR
			if(a%250==0) {
				RestoreTheScreen(theGIP,(int)seed1,(int)seed2,n_zero,a,
						num_n_zeroes, num_reps, 0);
			}
			if(a%ReportIteration==0 && a > 0) {
				for(j=0;j<ReportIteration;j++){
					printf(".");
				}
				fflush(stdout);
			}
			boing = kbhit();
			if(kbhit()) {
				RestoreTheScreen(theGIP,(int)seed1,(int)seed2,n_zero,a,
						num_n_zeroes, num_reps, 1);
				printf("\n\n********* PROGRAM MCLEEPS IS CURRENTLY SUSPENDED **********");
				printf("\nEnter any character to continue (i.e. a letter + RETURN key)\nOR");
				printf("\nSelect File->Quit or hit CMD-Q to quit\n");
				scanf("%s",&scrap_char);
				RestoreTheScreen(theGIP,(int)seed1,(int)seed2,n_zero,a,
						num_n_zeroes, num_reps, 0);
			}
#endif

			
			// this is the function within the hairy innards of which, the latent variables
			// get realized from $P^*(X)$.
			RealizeNew(theLIP,theGIP,theLP,n_zero);  
		  
		  	// This computes the terms (one for each locus) that get added to the Monte
		  	// Carlo average
			log_joint_multi_prob(theLIP,theGIP,theLP,n_zero);
		  
			if(a==0 && n_index==0) { // On the first time through, determine the LogNormFactor
									 // The LogNormFactor is a factor that gets multiplied to the
									 // likelihood so that it can be expressed in the machine without
									 // an underflow error.  At the end of averaging over all these 
									 // various quantities, it is divided out and the log-likelihood
									 // is reported
				LogNormFactor = (theLP->tot_joint - theLP->tot_draw)/ (double)num_loc;
				
				// Also, put a header on the columns in the outfile
				fprintf(likefile,"NUM_REPS=%d     \n",num_reps);
			}



			// update values in LocusAves and LocusVars, and compute TotalLike
			TotalLike = 1.0;  // get ready to accumulate a product
			TVTerm1 = 1.0;
			TVTerm2 = 1.0;

			for(j=0;j<num_loc;j++) { // update LocusAves via running mean recursion (S.M. Ross pg 116)
				theOldLocAv = LocusAves[j];
				tempLoc = theLP->joint[j] - theLP->draw[j];
				if (a > 1000)
					logLocHist[j][(int)floor(tempLoc+0.5) - logLocHistLow[j]]++;
				// on iteration 1000 use the current average to determine the histogram
				// the histogram includes 50 log units up and 150 log units down
				// from the log of the current average  likelihood for this locus
				else if (a == 1000)
					logLocHistLow[j] = (int)floor(log(LocusAves[j]) + LogNormFactor) - 150;
				LocusAves[j] += (exp(tempLoc - LogNormFactor) - LocusAves[j])/(a+1);
				TotalLike *= LocusAves[j];
				LocusSquares[j] += pow(exp(tempLoc - LogNormFactor),2.0);
				if (a > 0) {
					LocusVars[j] = (LocusSquares[j] - ((double)a+1.0)*LocusAves[j]*LocusAves[j])/
								   (double)(a * (a+1.0));
					TVTerm1 *= LocusAves[j]*LocusAves[j];
					TVTerm2 *= (-LocusVars[j] + (LocusAves[j]*LocusAves[j])); 
				}	
			}   // closes j-loop over loci
			
			TotVar = -TVTerm2 + TVTerm1;
			
		}	// closes the a-loop
		
		
		// print out the histograms to a different file
		outf = fopen("aa_LocusHistogramsOut.txt","a");
		for(j=0;j<num_loc;j++) {
			fprintf(outf,"Ne = %d and locus = %d\n", n_zero, j);
		for (k = 0; k < sizeOfHist; k++) {
			if (logLocHist[j][k] != 0)
				fprintf(outf,"%d\t", logLocHistLow[j] + k);
		}
		fprintf(outf,"\n");
		for (k = 0; k < sizeOfHist; k++) {
			if (logLocHist[j][k] != 0)
				fprintf(outf,"%d\t", logLocHist[j][k]);
		}
		fprintf(outf,"\n\n");
		}
		fclose(outf);
		
		// print out your estimated confidence intervals for the log-likelihood
		// This is based on Central Limit Theorem Normal approximation
		CIup =log(TotalLike + 1.96*sqrt(TotVar)) + theGIP->numLoci*LogNormFactor;
		if (TotalLike > 1.96*sqrt(TotVar)) {
			
			CIdown = log(TotalLike-1.96*sqrt(TotVar)) + theGIP->numLoci*LogNormFactor;
			TotalLike = log(TotalLike) + theGIP->numLoci*LogNormFactor;
			// ECA has changed these so it prints out as .3f's rather than .10e's (since they _are_ logs!)
			// ECA also has it print out the standard error because we should have that value there.
			fprintf(likefile, "%d\t%.3f\t%.3f\t%.3f\n",n_zero/2, 
				(double)TotalLike, (double)CIdown, (double)CIup  /*, sqrt(TotVar) */ );
//			printf("%d\t%.3f\t%.3f\t%.3f\t\n",n_zero/2, 
//				(double)TotalLike, (double)CIdown, (double)CIup /* , sqrt(TotVar) */);
		}
		else {
			TotalLike = log(TotalLike) + theGIP->numLoci*LogNormFactor;
			// ECA has changed these so it prints out as f's rather than e's (since they _are_ logs!)
			// ECA also has it print out the standard error because we should have that value there.
			fprintf(likefile,"%d\t%.3f\t-Inf\t%.3f\t\n",n_zero/2,TotalLike, CIup /*, sqrt(TotVar)*/ );
//			printf("%d\t%.3f\t-Inf\t%.3f\t\n",n_zero/2,TotalLike, CIup /* , sqrt(TotVar)*/ );
		}
		
		// print all that out on a locus by locus basis too:
		LocByLocPrint(LocusAves,LocusVars,n_zero,LogNormFactor);
		
		getsd(&seed_test1,&seed_test2);
				
		// close and open the likefile so that it saves the stuff in case the program bombs later on
		fclose(likefile);
		likefile = fopen("aa_LogLikelihoodsOut.txt","a");
		
		
	}	// closes the n_index loop
	
	fclose(likefile);
	free_dvector(LocusSquares,0,num_loc-1);
	
	// Print the finishing time out to the EchoFile.  Just use outf to point to it.
	// open it in append mode
	outf = fopen("aa_SettingsAndDataOut.txt","a");
	ATimeStruct = time(NULL);
	fprintf(outf,"\n\nProgram Completed at %s\n\n",ctime(&ATimeStruct));
	fclose(outf);
	
#ifdef MAC_CODE_WARRIOR
	// for the desktop version, we want to print out a completion note:
	printf("\n\n\n********* Program MCLEEPS has completed its run **********");
	printf("\n\nOutput is in the files:");
	printf("  \n                         aa_SettingsAndDataOut.txt");
	printf("  \n                         aa_LogLikelihoodsOut.txt");
	printf("  \n                         aa_LocByLocLogLOut.txt");
	printf("  \n                         aa_LocusHistogramsOut.txt\n\n");
	printf("An explanation of the output is given with the program documentation.");
	printf("\n\nHave a nice day.\n");
#endif 
	

	
	return(0);
}


/*********************************************/
/* buffer.C: the latest version... */

CBuffer::CBuffer()
{
  mBuffer = new char[MAX_BUF];
  mIsEndInBuffer = FALSE;
  mIsAtEnd = FALSE; 
}

CBuffer::CBuffer(char *inFileName)
{

  mBuffer = new char[MAX_BUF];
  mIsEndInBuffer = FALSE;
  mIsAtEnd = FALSE; 
  mInFile.open(inFileName);
  if ( mInFile == NULL) {
    printf("\nSorry...can't find file named %s",inFileName);
    printf("\nPlease check your input file name and restart the program");
    printf("\n...Exiting to System now...\n\n");
    #ifdef MAC_CODE_WARRIOR
    printf("\n\nHit CMD-Q to Quit, or choose File->Quit.  Then relaunch the program\n\n");
    #endif
    exit(1);
  }  
  FillBuffer();
  mInFile.close();
}	

void
CBuffer::OpenBuffer(char *inFileName)
{
  mIsEndInBuffer = FALSE;
  mIsAtEnd = FALSE; 
  mInFile.open(inFileName);
  if ( mInFile == NULL) {
    cout << "Not a valid file name!" << endl;
    exit(1);
  }
  FillBuffer();
  mInFile.close();
}	

void
CBuffer::CloseBuffer()
{
  delete [] mBuffer;
}

CBuffer::~CBuffer()
{
  delete [] mBuffer;
}


// A NEW VERSION:
char
CBuffer::GetNextByte(void)
{
  char theNextByte;

  theNextByte = mBuffer[mBufPos++];
  if (mBufPos == mBufSize) { 
		if(mIsEndInBuffer)
			mIsAtEnd = E_TRUE;
	}
	// definately messed up!
	if (mBufPos > mBufSize) {
		cout << "ERROR Went too far!" << endl;
	  exit(1);
	}
	while (theNextByte == '#') {
		while ((theNextByte != '\n') && (theNextByte != '\r') && !mIsAtEnd) 
			theNextByte = mBuffer[mBufPos++];
		theNextByte = mBuffer[mBufPos++];
	}
  return theNextByte;
}


void
CBuffer::FillBuffer()
{
  mBufPos = 0L;
  mBufSize = 0L;
  // just make it grab the whole file every time...
  while (mInFile.peek() != EOF) {
    mInFile.get(mBuffer[mBufSize++]);
    if (mBufSize == MAX_BUF)
      exit(1);
  }
  mIsEndInBuffer = TRUE;        
  mBuffer[mBufSize] = '\r';    
}
#if 0 // the old version...
void
CBuffer::fillBuffer(void)
{
  FILE *InFile;
  char ch;
  char file_name[MAX_NAMELEN];

  mBufPos = 0; /* so the way it is now, you can't start Jan 1 */
  InFile = buf_file;
  if (InFile == NULL) {
    printf("Bad file name in file_name[]!\n");
    exit(1);
  }
  if (file_pos == 0) {
    fseek(InFile, file_pos, 0);
    mBufSize = fread(mBuffer, 1, myBufSize, InFile);
  }
  else {
    fseek(InFile, file_pos, 0);
    mBufSize = fread(mBuffer, 1, MIN_BUF, InFile);
  }
  if (mBufSize == myBufSize) {
    while ((ch = (char)getc(InFile)) != '\n') {
      if (mBufSize > myBufSize + EXTRA_BUF) {
        printf("Error in fillBuffer() \n");
        exit(1);
      }
      mBuffer[mBufSize] = ch;
      if (ch == EOF)
        break;
      else
				mBufSize++;
    }
    mBuffer[mBufSize] = ch;
  }
  else
    mBuffer[mBufSize] = EOF;
  if (mBuffer[mBufSize] != EOF) {
    mBuffer[++mBufSize] = (char)NULL;
    file_pos = ftell(InFile);
  }
  fclose(InFile);
}
#endif

void
CBuffer::SkipSpace(void)
{
  while ((isspace(InspectByte(0)) != FALSE) && (!mIsAtEnd))
    GetNextByte();
}

/* find the next non-space and then skip to the next space */
void
CBuffer::SkipOneCol(void)
{
  char theCh=0;  // initialize so it doesn't produce a warning

  if (!mIsAtEnd)
    theCh = GetNextByte();
  while ((theCh != '\n') && (theCh != '\t') &&
         (theCh !=  '\r') && (theCh != ' ') && !mIsAtEnd) 
    theCh = GetNextByte();
  SkipSpace();
}

void
CBuffer::SkipCols(long num_cols)
{
  long I = 0;

  for (I = 0; I < num_cols; I++)
    SkipOneCol();
}
void
CBuffer::MoveToNextRow(void)
{
  char theCh=0;  // initialize so it doesn't produce a warning
  if (!mIsAtEnd)
    theCh = GetNextByte();
  while ((theCh != '\n') && (theCh != '\r') && !mIsAtEnd) 
    theCh = GetNextByte();
}

void
CBuffer::SkipRows(long num_rows)
{
  long I;

  for (I = 0; I < num_rows; I++) 
		MoveToNextRow();
}


// HERE IS A NEW VERSION FROM ELLEN FOR ALLOWING COMMENTS IN THE INPUT FILE
double
CBuffer::GetFloat(void)
{
  char *theFltPtr;
  double theFlt;
 char theNextByte;
  
	SkipSpace();
	theNextByte = mBuffer[mBufPos];
	while (theNextByte == '#') {
		while ((theNextByte != '\n') && (theNextByte != '\r') && !mIsAtEnd) 
			theNextByte = mBuffer[mBufPos++];
		theNextByte = mBuffer[mBufPos];
	}
	theFltPtr = &(mBuffer[mBufPos]);
  theFlt = strtod(theFltPtr,(char **)0);
  SkipOneCol();
  return (double)theFlt;
}


float
CBuffer::InspectFloatPast(long inBytesToSkip)
{
  char *flt_ptr;
  double flt;
  long space_bytes;

  space_bytes = 0;
  while (isspace(mBuffer[mBufPos + inBytesToSkip + space_bytes]))
    space_bytes++;
  flt_ptr = &mBuffer[mBufPos + inBytesToSkip + space_bytes];
  flt = atof(flt_ptr);
  return (float)flt;
}


/* retreives the byte incr increments beyond the current position */
/* in the mBuffer but does not advance mBufPos */
/* or retreives the byte -incr increments before the current position */
/* in the mBuffer but does not change mBufPos */
char
CBuffer::InspectByte(long incr)
{
  if (((mBufPos + incr) > mBufSize) || ((mBufPos + incr) < 0)) {
    printf("Error in Buffer::InspectByte(), can't look beyond end of mBuffer!\n");
    exit(1);
  }
  return mBuffer[mBufPos + incr];
}


long
CBuffer::BytesToNextCol(long inBytesToSkip)
{
  long more_bytes;

  more_bytes = 0;
  while (isspace(InspectByte(inBytesToSkip + more_bytes)))
    more_bytes++;
  while (!isspace(InspectByte(inBytesToSkip + more_bytes)))
    more_bytes++;
  while (isspace(InspectByte(inBytesToSkip + more_bytes)))
    more_bytes++;
  return more_bytes;
}

/* note: each tide entry = 1 row */
long
CBuffer::BytesToRow(long row_num)
{
  char ch;
  long I;
  long K;

  I = 0;
  if (row_num > 0) {
    for (K = 1; K < row_num; K++) {
      ch = InspectByte(I++);
      while (ch != '\n')
        ch = InspectByte(I++);
    }
  }
  else {
    for (K = 2; K > row_num; K--) {
      ch = InspectByte( I--);
      while (ch != '\n')
        ch = InspectByte( I--);
    }
    I+=2;
  }
  return I;
}


// counts the rest of the columns in the row without advancing mBufPos

// this version (from Ellen August 18,2000) counts columns but doesn't count "empty columns"
// in MS Excel if you generate a tab delimited file you usually
// wind up with "bonus" tabs at the end of shorter lines (rows)
// this caused the former CountCols to give you an overestimate of the actual 
// number of columns in files derived from Excel unless you manually
// removed all the "bonus" tabs, which was a major pain
// note: a blank column in the middle of a row will also not be counted now
// so if you want it counted you have to put something there, any non-tab
// char will do the job
/*
int
CBuffer::CountCols(void)
{
  int theColCount = 0;
  int theByteInspected = 0;
  char theCh = InspectByte(theByteInspected);
 
  while ((theCh != '\n') && (theCh != '\r')) {
    while ((theCh != '\t') && (theCh != '\n') && (theCh != '\r'))
      theCh = InspectByte(theByteInspected++);
    while (theCh == '\t')
      theCh = InspectByte(theByteInspected++);
    theColCount++;
  }

  return theColCount;
}
*/


// Here is a CountCols that should treat any whitespace as a signal to count a new
// column (but not multiple white spaces stuck together.  It seems to work.  This
// is much preferable, methinks.  
int
CBuffer::CountCols(void)
{
  int theColCount = 0;
  int theByteInspected = 0;
  char theCh = InspectByte(theByteInspected);
 
  while ((theCh != '\n') && (theCh != '\r')) {
    while ((theCh != '\t') && (theCh != '\n') && (theCh != '\r') &&
           (theCh != ' '))
      theCh = InspectByte(theByteInspected++);
    while (theCh == '\t' || theCh == ' ')
      theCh = InspectByte(theByteInspected++);   
    theColCount++;
  }

  return theColCount;
}


char *
CBuffer::GetString(void)
{
  long theMaxStrLen = 300; // change both of these if you need to change the 300
  char theString[300]; 
  long I = 0;
  SkipSpace();

  theString[I++] = GetNextByte();
  while ((theString[I - 1] != '\n') &&
         (theString[I - 1] != '\r') &&
         (I < theMaxStrLen))
         theString[I++] = GetNextByte();
  theString[I - 1] = '\0';       
  char *theNewString = new char[I]; 
  for (long J = 0; J < I; J++)
    theNewString[J] = theString[J];
  return theNewString;         
}

unsigned char *
CBuffer::GetPascalString()
{
  long theMaxStrLen = 300; // change both of these if you need to change the 300
  unsigned char theString[300]; 
  long I = 0;
  SkipSpace();

  theString[++I] = (unsigned char)GetNextByte();
  while ((theString[I] != '\t') &&
         (theString[I] != '\n') &&
         (theString[I] != '\r') &&
         (theString[I] != '\0') &&
         (I < theMaxStrLen))
         theString[++I] = (unsigned char)GetNextByte();
  theString[I] = '\0';
  theString[0] = (unsigned char)(I - 1); // don't include the terminal null
  unsigned char *theNewString = new unsigned char[I + 1]; // make room for first byte + null
  for (long J = 0; J <= I; J++)
    theNewString[J] = theString[J];
  return theNewString;         
}

// HERE IS A NEW VERSION FROM ELLEN FOR HANDLING COMMENTS
int
CBuffer::GetInt(void)
{
  char *theIntPtr;
  int theInt;
  char theNextByte;
  
	SkipSpace();
	theNextByte = mBuffer[mBufPos];
	while (theNextByte == '#') {
		while ((theNextByte != '\n') && (theNextByte != '\r') && !mIsAtEnd) 
			theNextByte = mBuffer[mBufPos++];
	    theNextByte = mBuffer[mBufPos];		
	}
  theIntPtr = &(mBuffer[mBufPos]);
  theInt = atoi(theIntPtr);
  SkipOneCol();
  return theInt;
}


// Classes.C
ListEntry::ListEntry()
{
  nxt = NULL;
}

SpecialListEntry::SpecialListEntry()
{
  nxt = NULL;
}

Locus  & Locus::operator=(Locus &inL)
{
  int i;
  delete [] val;
  delete [] notLost;
  numAlleles = inL.numAlleles;
  sz = inL.sz;
  val = new int[numAlleles];
  notLost = new int[numAlleles];
  for(i=0;i<numAlleles;i++) {
    val[i] = inL.val[i];
    notLost[i] = inL.notLost[i];
  }
  return(*this);
}


void Locus::init(int inNumAlleles)
{
  int i;    // temp variable for looping over val values and setting to zero
  sz = 0; 
  numAlleles = inNumAlleles;
  val = new int[inNumAlleles];
  notLost = new int[inNumAlleles];
  // even though new probably initializes things to zero.  ECA is going to make sure of it
  for(i=0;i<inNumAlleles;i++) {
  	val[i] = 0;
  }
  
}

Locus::~Locus()
{
  delete [] val;
  delete [] notLost;
}

GeneInfo::GeneInfo(void)
{
	CBuffer *theBuf;
	int I, J, K, L;
	theBuf = new CBuffer((char *)gINFILE);
	numReps = theBuf->GetInt();
	nLow = theBuf->GetInt();
	nHigh = theBuf->GetInt();
	nIncr = theBuf->GetInt();
	numLoci = theBuf->GetInt();
	numSamples = theBuf->CountCols();
	gen = new int[numSamples];
	for (I = 0; I < numSamples; I++)
	  gen[I] = theBuf->GetInt();		
	T = gen[numSamples - 1];
	sample = new Sample[T+1];
	for (I = 0; I <= T; I++)	
		sample[I].locus = new Locus[numLoci];
	theBuf->SkipSpace();
	for (I = 0; I < numLoci; I++) {
		L = 0;
		for (J = 0; J <= T; J++) {
			if (J  == 0)
				sample[J].locus[I].init(theBuf->CountCols());
			else
				sample[J].locus[I].init(sample[0].locus[I].numAlleles);	
			if (gen[L] == J) {	
				for (K = 0; K < sample[J].locus[I].numAlleles; K++) {
					sample[J].locus[I].val[K] = theBuf->GetInt();
					sample[J].locus[I].sz += sample[J].locus[I].val[K];
				}
				L++;
			}
			else {
				for (K = 0; K < sample[J].locus[I].numAlleles; K++)
					sample[J].locus[I].val[K] = 0;
			}
		}
	theBuf->SkipSpace();
	}
	delete theBuf;
	for (I = 0; I < numLoci; I++) {
		for (K = 0; K < sample[0].locus[I].numAlleles;  K++) {
			for (J = T; J >= 0; J--) {
				if (sample[J].locus[I].val[K] == 0)
				  sample[J].locus[I].notLost[K] = 0;
				else {
					for (int S = 0; S <= J; S++)
						sample[S].locus[I].notLost[K] = 1;
				  break;	
				}
			}
		}
	}
}


GeneInfo::~GeneInfo()
{
  for (int I = 0; I <= T; I++) 
		delete [] sample[I].locus;
  delete [] sample;
  delete [] gen;		
}


// a new SortAlleles
void
GeneInfo::SortAlleles()
{

	int t, j, k;
	int theNLRank[gMAX_ALLELES], theMeanFreq[gMAX_ALLELES];
	int *theOrder;
	
	// eric dynamically declared this stuff:
//	int (theOrigSample[gMAX_SAMPLES])[gMAX_ALLELES];
//	int (theOrigNL[gMAX_SAMPLES])[gMAX_SAMPLES];

	int **theOrigSample, **theOrigNL;
	
	theOrigSample = imatrix(0,gMAX_SAMPLES,0,gMAX_ALLELES);
	theOrigNL = imatrix(0,gMAX_SAMPLES,0,gMAX_ALLELES);

	
	for (j = 0; j < numLoci; j++) {  // cycle over loci
		for (k = 0; k < sample[0].locus[j].numAlleles;  k++) { // cycle over alleles
		  (theOrigSample[0])[k] = sample[0].locus[j].val[k];
		  (theOrigNL[0])[k] = sample[0].locus[j].notLost[k];		  
		  theNLRank[k] = sample[0].locus[j].notLost[k];
		  theMeanFreq[k] = 100*sample[0].locus[j].val[k];
		  for (t = 1; t <= T; t++) { // cycle through all the generations
		    (theOrigSample[t])[k] = sample[t].locus[j].val[k];
		    (theOrigNL[t])[k] = sample[t].locus[j].notLost[k];		  		    		  
		    theNLRank[k] += sample[t].locus[j].notLost[k];
		    theMeanFreq[k] += 100*sample[0].locus[j].val[k];
		  } // end generation loop		
		} // end allele loop
		// rank alleles as follows: 100*(number of generations not lost) + (mean frequency * 100)
		// this way alleles are first sorted by how long they stick around, and then
		// sub-sorted by their frequency
		for (k = 0; k < sample[0].locus[j].numAlleles;  k++) { // cycle over alleles
			theNLRank[k] *= 100;
			theNLRank[k] += theMeanFreq[k]/T;
		}
		// now sort the alleles for this locus
		theOrder = SpecialSort(theNLRank, sample[0].locus[j].numAlleles);
		for (k = 1; k < sample[0].locus[j].numAlleles;  k++) { // cycle over alleles
		  for (t = 0; t <= T; t++) { // cycle through all the generations
		    // set everything for each allele equal to the info for the allele
		    // will now occupy this slot, reversing the order of alleles so that
		    // the secondmost common allele gets to be first... on down to the least common allele, but
		    // reserving the very last slot for the most common allele
		  	sample[t].locus[j].val[k-1] = (theOrigSample[t])[theOrder[sample[0].locus[j].numAlleles - k - 1]];
		  	sample[t].locus[j].notLost[k-1] = (theOrigNL[t])[theOrder[sample[0].locus[j].numAlleles - k - 1]];		  	
		  } // end generation loop		
		} // end allele loop
		// get that last allele, the most common one, into the last slot
		for (t = 0; t <= T; t++) { // cycle through all the generations
		  	sample[t].locus[j].val[sample[0].locus[j].numAlleles - 1] = (theOrigSample[t])[theOrder[sample[0].locus[j].numAlleles - 1]];
		  	sample[t].locus[j].notLost[sample[0].locus[j].numAlleles - 1] = (theOrigNL[t])[theOrder[sample[0].locus[j].numAlleles - 1]];		  	
		} // end generation loop	
		delete [] theOrder;
	} // end loci loop
}


void GeneInfo::PrintAll(void)
{
  int I, J, K;
	cout << "Doing " << numReps << " iterates" << endl;
  cout << "Samples taken at generations ";
	for (I = 0; I < numSamples; I++)
		cout << gen[I] << "\t";
  cout << endl;
	cout << "Computing the likelihood for population sizes between ";
	cout << nLow << " and " << nHigh << " in increments of " << nIncr << endl;
	for (I = 0; I < numLoci; I++) {
		for (J = 0; J <= T; J++) {
			for (K = 0; K < sample[0].locus[I].numAlleles;  K++) {
				cout << sample[J].locus[I].val[K] << "\t";
			}
			cout << endl;
		}
		cout << endl;
	}
	cout << endl;
}


// AN ECA ADDITION:
LatentInfo::LatentInfo(GeneInfo *G)
{
	int t;
 
	T = G->gen[G->numSamples - 1];
	year = new Sample[T+1];
	numLoci = G->numLoci;
	
	for(t=0;t<=T;t++)  {
		year[t].locus = new Locus[G->numLoci];
		 
		for (int i = 0; i < G->numLoci; i++) {
		  year[t].locus[i].init(G->sample[0].locus[i].numAlleles);
		  for (int j = 0; j < year[t].locus[i].numAlleles; j++)
		    year[t].locus[i].val[j] = 0;
		}
	}
}



void
LatentInfo::FreqArray(int t, int j, double *p, int n_zero) 
{   
	for (int k = 0; k < year[t].locus[j].numAlleles;k++)
		p[k] = Freq(t,j,k,n_zero);
}

int 
LatentInfo::SumRemaining(int t, int j, int k) 
{ 
	int s=0;
	for(int i=k+1;i<year[t].locus[j].numAlleles; i++)
		s += year[t].locus[j].notLost[i];
	return(s);
}


/* This member function looks at the *samples only* and sets the NotLost elelments of a 
LatentInfo struct to appropriate starting values at the beginning of a call to the function
RealizeNew.  Those values will be updated when
the LatentInfo struct goes through the RealizeNew function.  
*/
void LatentInfo::RefreshNotLost(GeneInfo *inG) 
{
	int I, J, K, S;
	 for (I = 0; I < inG->numLoci; I++) {
		for (K = 0; K < inG->sample[0].locus[I].numAlleles;  K++) {
			for (J = inG->T; J >= 0; J--) {
				if (inG->sample[J].locus[I].val[K] == 0) {                 // &&  year[J].locus[I].val[K] == 0)
					year[J].locus[I].notLost[K] = 0;  // ECA changed FALSE to 0
				  									// so if the allele could be lost it gets a zero
				}
				else {
					for (S = 0; S <= J; S++)
						year[S].locus[I].notLost[K] = 1;  // ECA changed TRUE to 1
				  										// so if the allele cannot be lost it gets a one
					break;	
				}
			}
		}
	}
}

// AN ECA ADDITION:
LatentInfo::~LatentInfo()
{
  for (int I = 0; I <= T; I++) 
		delete [] year[I].locus;
}

LogProb::LogProb(int inNumLoci)
{
  draw = new double[inNumLoci];
  joint = new double[inNumLoci];  
}

/*LogProb::LogProb(GeneInfo *inG)
{
  draw = new double[inG->numLoci];
  joint = new double[inG->numLoci];  
} */

void
LogProb::SetDrawToZero(int inNumLoci) 
{
	tot_draw = 0.0;
	for(int j=0;j<inNumLoci;j++)
		draw[j] = 0.0;
}


LogProb::~LogProb()
{
  delete [] draw;
  delete [] joint;  
}

PopInfo::PopInfo(int inT, int inN)
{
	T = inT;
	psz = new int[inT];
	for (int i = 0; i < inT; i++)
		psz[i] = inN;		
}

PopInfo::~PopInfo()
{
	delete [] psz;
}
void PopInfo::PutConstPsz(int inN)
{
	for (int i = 0; i < T; i++)
		psz[i] = inN;		
}


PopInfo & PopInfo::operator=(PopInfo &inPopInfo)
{
	delete [] psz;
	T = inPopInfo.T;
	psz = new int[T];
	for(int i =0;i<T;i++) 
		psz[i] = inPopInfo.psz[i];
	return(*this);
}


StoredInfo::StoredInfo(GeneInfo *inG)
{
	Prev_Av = new double[inG->numLoci];
	LI = new LatentInfo(inG);
	LP = new LogProb(inG->numLoci);
	theLogRatios = new double[inG->numLoci];
}


// Ne_Util.C

long
AChooseB(long A, long B)
{
  double temp;
  temp = 1.0;
  long I;
  if (B > A)
    return 0L;
  for (I = A; I > B; I--)
    temp *= (double)I/(I-B);
	if (temp > pow(2.0,31))
		cout << "TOO MANY CONFIGURATIONS!!!!!" << endl;
  return (long)floor(0.0001+temp);
}

// returns the index from a list of doubles of the largest value
long
FindMax(double *theList, long theLen)
{
  long I, theMax;
  theMax = 0;
  for (I = 0; I < theLen; I++)
    if (theList[I] > theList[theMax])
      theMax = I;
  return theMax;
}
 
void
Sort(double *inArray, long inSize)
{
	ListEntry *theListElements = new ListEntry[inSize];
  ListEntry *theFirst, *theTemp, *theLastTemp;
	long I;
	theFirst = NULL;
  for (I = 0; I < inSize; I++) {
		theListElements[I].val = inArray[I];
		theLastTemp = NULL;
		theTemp = theFirst;
		while ((theTemp != NULL) && (theTemp->val < theListElements[I].val))  {
			theLastTemp = theTemp;
			theTemp = theTemp->nxt;
		}
    theListElements[I].nxt	= theTemp;
		if (theLastTemp == NULL)
			theFirst =  &(theListElements[I]);
    else
  		theLastTemp->nxt = &(theListElements[I]);
	}
	theTemp = theFirst;
	I = 0;
	while (theTemp != NULL) {
		inArray[I++] = theTemp->val;
		theTemp = theTemp->nxt;
	}
  delete [] theListElements;
}

// right now this function doesn't re-order inArray, it just returns the list in which
// each entry represents the index in inArray that should go in that slot, 1st slot  being
// for the smallest element
int *
SpecialSort(int *inArray, int inSize)
{
  int *thePermutation = new int[inSize];
  SpecialListEntry *theListElements = new SpecialListEntry[inSize];
  SpecialListEntry *theFirst, *theTemp, *theLastTemp;
  int I;
  theFirst = NULL;
  for (I = 0; I < inSize; I++) {
		theListElements[I].val = inArray[I];
		theListElements[I].orig = I;
		theLastTemp = NULL;
		theTemp = theFirst;
		while ((theTemp != NULL) && (theTemp->val < theListElements[I].val))  {
			theLastTemp = theTemp;
			theTemp = theTemp->nxt;
		}
    theListElements[I].nxt	= theTemp;
		if (theLastTemp == NULL)
			theFirst =  &(theListElements[I]);
    else
  		theLastTemp->nxt = &(theListElements[I]);
  }
  theTemp = theFirst;
  I = 0;
  while (theTemp != NULL) {
  	thePermutation[I++] = theTemp->orig;
	theTemp = theTemp->nxt;
  }
  delete [] theListElements;
  return thePermutation;
}

/*************
*  FUNCTION:  double draw_folded_asin_norm(double mean,double var, int n, int no_zero, 
										int save_slots, int *x, double schmear_prob)
*  PURPOSE:   Generate a random integer between appropriate low and high bounds which
			  are dictated by the values of no_zero and save_slots.  This function returns
			  the corresponding theta as a double (this is then used in adjusting the distribution
			  one step back in time during the backwards phase.  The integer x is returned as an 
			  output parameter.
*
*  INPUT:     int mean -->  the mean of the normal distribution to draw from
			  int var  -->  the variance of the normal distribution to draw from
			  int n    -->  the population size 
*			  int no_zero  -->  1 if the allele cannot go extinct, 0 if it can
*  			  int save_slots  -->  the number of remaining allele types that must have at least
								   one copy at this time point in the final realization of x.
								   
			  double schmear_prob  -->  This is a value useful if you want to just take a uniform value some
			  			  				portion of the time.  I (ECA) originally thought that might 
			  			  				help the importance sampling, but actually doesn't.  Hence, best to
			  			  				leave its value at 0.0 all the time.  
								   
*  OUTPUT:	  int *x  -->  the discrete value that the latent variable x takes.



		ECA:  THIS VERSION IS SLIGHTLY MODIFIED FROM THE ORIGINAL---I HAVE MADE SURE THAT 
		THERE IS EXACTLY ONE CALL TO GENNOR EACH TIME THIS FUNCTION IS CALLED.  THIS
		HELPS TO ENSURE THAT WHEN WE TRY COUPLING THINGS, WE GET THE RESULTS THAT
		WE WOULD LIKE TO GET.

*************/


double 
draw_folded_asin_norm(double mean,double var, int n, int no_zero, int save_slots, int *x, double schmear_prob)
{
	
	double theta;
	double lb,hb;  // the low bound and high bound
	double r;
	
	
	// FIRST THINGS FIRST WE DRAW THE VALUE OF THETA.  THIS ENSURES THAT REGARDLESS OF THE STATUS OF
	// SAVE_SLOTS OR NO_ZERO, WE WILL ALWAYS MAKE ONE CALL TO gennor() FOR EACH CALL OF THIS FUNCTION.
	theta = (double)gennor((float)mean,(float)sqrt(var));
	
	// draw a uniform 0,1 deviate and if it falls between .4 and .4 + schmear_prob, then we just draw an 
	// x at uniform from the range of possible values and return the appropriate info.
	// I make the little window between .4 and .4+schmear_prob, because, quite frankly, I don't trust
	// that the ranf function isn't a little weird near the ends.
	r = ranf();
	if(r>.4 && r < .4 + schmear_prob) {
		*x = uniform(no_zero, n-save_slots);
		return(asin(sqrt((double)(*x)/(double)n)));
	}
	
	// if r wasn't in that little window we got down here.
	// report any errors if they are about to happen
	if(n < save_slots) {
			printf("\n\nnn < save_slots in draw_folded_asin_norm()\n...Exiting program...\n\n"); 
			exit(1);
			*x = 0;
			return(0.0);
	}
	// first deal with the special case when n == save_slots.  In this case, the realized allele
	// count must be zero (and so its probability will be one)
	if(n == save_slots) {
		if(no_zero == 1) {
			printf("\n\nno_zero is 1 when n == save_slots in draw_folded_asin_norm() \n\n...Crashing and burning...\n\n"); 
			printf("\n\n...Exiting to system...");
			exit(1);
		}
		*x = 0;
		return(0.0);
	}
	
	
	
	// determine the values of the bounds
	if(save_slots > 0) { // if save_slots > 0 the following is well-defined
		hb = asin(sqrt( ((double)n - (double)save_slots + .5 )/(double)n)  );
	}
	else {   // works out because draw prob is just the integral of the density to the right of hb
		hb = asin(sqrt( ((double)n - .5 )/(double)n)  );
	}
	lb = asin(sqrt(.5/(double)n));
	
	if(no_zero == 1 && save_slots >= 1)  {  // allele can netiher go extinct nor fix at freq one in the
											// population so we must fold the dsn around lb or hb
		if( theta < lb )   {
			theta = 2.0 * lb - theta;    // reflect theta through the lower bound
			while(theta > hb) {   // if the reflected theta turns out to be greater than the high bound
			                      // then translate it to the left as many times as it needs to be for it
			                      // to finally by less than hb
				theta = lb + (theta - hb);
			}  // by this time theta is guaranteed to be lower than the higher bound
			   // and so will make it through the following condtionals (we hope!)
		}   
		if( theta >= hb) {
			theta = 2.0 * hb - theta;  // in this case we must reflect theta through the high bound
			while(theta < lb) {   // if the reflected theta turns out to be lower than the low bound
			                      // then translate it to the right as many times as it needs to be for it
			                      // to finally by more than lb
				theta = hb - (lb - theta);
			}  // by this time theta is guaranteed to be higher than the lower bound
		}
		// send back the results
		*x = (int)(  pow(sin(theta),2) * (double)n + .5);
		return(theta);
	}
	
	if(no_zero == 0 && save_slots >= 1) {  // allele count is allowed to be zero but some space for
										   // extra alleles must remain
		if( theta < lb )   {
			theta = 0.0;    // set theta to zero (this will affect the conditional distribution in the
							// previous time step---which is the next distribution we will draw from
							// in the backward cycle.)
		}   
		if( theta >= hb) {
			theta = 2.0 * hb - theta;  // in this case we must reflect theta through the high bound
			while(theta < lb) {   // if the reflected theta turns out to be lower than the low bound
			                      // then translate it to the right as many times as it needs to be for it
			                      // to finally by more than lb
				theta = hb - (lb - theta);
			}  // by this time theta is guaranteed to be lower than the lower bound
			   // and so will make it through the following condtionals (we hope!)
		}
		// send back the results
		*x = (int)(  pow(sin(theta),2) * (double)n + .5);  // note that if theta = 0.0, this will return
														   // *x = 0.  
		return(theta);	
	}
	
	if(no_zero == 0 && save_slots == 0) {  // allele count is allowed to be zero or to fix at freq one
		if( theta < lb )   {
			theta = 0.0;    // set theta to zero (this will affect the conditional distribution in the
							// previous time step---which is the next distribution we will draw from
							// in the backward cycle.)
		}   
		if( theta >= hb) {
			theta = PI/2.0;
		}
		// send back the results
		*x = (int)(  pow(sin(theta),2) * (double)n + .5);  // note that if theta = 0.0, this will return
														   // *x = 0, and if theta = PI/2.0 this will
														   // return *x = n
		return(theta);	
	}
	
	if(no_zero == 1 && save_slots == 0)  {  // allele can't go extinct but can fix at freq one
		if( theta < lb )   {
			theta = 2.0 * lb - theta;    // reflect theta through the lower bound
			while(theta > hb) {   // if the reflected theta turns out to be greater than the high bound
			                      // then translate it to the left as many times as it needs to be for it
			                      // to finally by less than hb
				theta = lb + (theta - hb);
			} 
		}   
		if( theta >= hb) {
			theta = PI/2.0;
		}
		// send back the results
		*x = (int)(  pow(sin(theta),2) * (double)n + .5);  // note that if theta = PI/2.0 this will
														   // return *x = n
		return(theta);	
		
	}
	
	
	// if anything gets down to here, something is horribly amiss.  Tell the user so and return a value that will 
	// make everything bomb.
	printf("\n\nProgram Error!  Got through all the conditionals in draw_folded_asin_norm().");
	printf("\nPrepare to crash and burn\n\n");
	printf("\n...Exiting to System...\n");
	exit(1);
	return(-55.0);
	
}      



/*************
*  FUNCTION:  log_joint_multi_prob(LatentInfo *inX, GeneInfo *inY, LogProb *inLP, int n_zero)
*  PURPOSE:   Compute the joint probability of the latent variables (X) and the observed variables
			  (Y).  
			  
			  The sequence of ifs and thens don't necessarily follow the presentation in the 
			  appendix of the Genetics article---I wrote that after thinking about how it could
			  be implemented more cleanly.  

*
*  INPUT:     LatentInfo *inX  -->  Pointer to the struct that holds the information about the
								    latent allele freqs
			  GeneInfo *inY  -->  Pointer to the struct that holds the information about the observed
			  					     allele counts.
			  LogProb *inLP  --> Pointer to a struct which contains arrays where the result will go
			  
			  int n_zero     --> The effective population size currently assumed.  
*************/




void 
log_joint_multi_prob(LatentInfo *inX, GeneInfo *inY, LogProb *inLP, int n_zero)
{
	int t,j;
	double locus_log_prob;
	double s[gMAX_GEN];
	double p[gMAX_ALLELES];
	
	inLP->tot_joint = 0.0;	
	
	for(j=0;j<num_loc;j++)  {  // cycle over loci
		locus_log_prob = 0.0; //  initialize to accumulate a sum
		for(t=0;t<num_gen;t++) { // cycle over to store sample sizes in a short-named variable
			s[t] = inY->SSZ(t,j);
		}
		for(t=0;t<num_gen;t++)  {  // cycle over generations
			if(t == 0) {  // compute the prior probability which is one over the number of possible
						  // allelic configurations which are polymorphic (this is C-{poly} from Ellen's
						  // paper in Genetics.  The expression from Feller is:
				locus_log_prob += LogFactorial(inY->sample[0].locus[j].numAlleles - 1) +
								  LogFactorial(n_zero) - LogFactorial(n_zero + inY->sample[0].locus[j].numAlleles - 1);
			}
			else {
				inX->FreqArray(t-1,j,p,n_zero); 
				locus_log_prob += log_multi_prob(n_zero, p, inX->XArray(t,j), inX->year[t].locus[j].numAlleles);

			}
			if(s[t] > 0 ) {   // compute the sample probability
				inX->FreqArray(t,j,p,n_zero); 
				locus_log_prob += log_multi_prob((int)(s[t]+.001), p, inY->YArray(t,j), inY->sample[t].locus[j].numAlleles);
			}
		} // closes the t-loop
		inLP->joint[j] = locus_log_prob;
		inLP->tot_joint += locus_log_prob;
	} // closes the j-loop
	
	
}



/*************
*  FUNCTION:  double log_multi_prob(int n, double *p, int *x, int inNum)
*  PURPOSE:   Return the log of the multinomial probability function
*
*  INPUT: 	  int n  -->  number of trials
			  double *p  --> the probability of success
*  			  int *x  -->  the number of successes
			  int inNum --> the number of categories
*************/



double 
log_multi_prob(int n, double *p, int *x, int inNum)
{
	int k;  // subscript over categories
	double log_prob; 
	int check_n;
	double check_p;
	
	
	log_prob = LogFactorial(n);  // initialize to log n! to accumulate the sum
	check_n = 0;
	check_p = 0.0;
	
	for(k=0;k<inNum;k++) {
		if(p[k] != 0 ) {
			log_prob += x[k] * log(p[k]) - LogFactorial(x[k]);
		}
		else if(x[k] != 0) {
			printf("\n\nYou're bumming my friend.  You have a zero probability event \nin log_multi_prob!\n\n");	
			printf("\nCrashing and burning\n");
			printf("\n...Exiting to system...\n");
			exit(1);
		}
		check_n += x[k];
		check_p += p[k];
	}
	
	if(check_n != n) {
		printf("\n\nSum of x's is not n in log_multi_prob\n\n");
		printf("\nThis cannot be tolerated!!  Get me out of here!");
		printf("\n...Exiting to system...\n");
		exit(1);
	}
	if(check_p < .99999999999 || check_p > 1.00000000001) {
		printf("\n\nSum of p's is not 1.0 in log_multi_prob\n\n");
		printf("\nThis is a grave problem!!  Going away now!");
		printf("\n...Exiting to system...\n");
		exit(1);
	}
	return(log_prob);
}


/*************
*  FUNCTION:  double asin_draw_prob(double mean, double var, int low, int high, int x)
*  PURPOSE:   ArcSine Square-root transformed NORM(al) DRAW PROB(ability)
*			  Compute the probability of having drawn x using gennor to get the theta and
			  then transforming that back into a frequency and then discretizing that into
			  an x given the mean and variance of the dsn for theta and n, the pop size and all
			  the reflections and translations that were necessary given the values of no_zero
			  and save_slots 
			  
			  The sequence of ifs and thens don't necessarily follow the presentation in the 
			  appendix of the Genetics article---I wrote that after thinking about how it could
			  be implemented more cleanly.  

*
*  INPUT:     double mean -->  the mean of the normal distribution drawn from
			  double var  -->  the variance of the normal distribution drawn from
*  			  int x  -->  the realized x value 
			  int n  -->  the effective population size in that generation.
			  int no_zero -->  takes '1' if allele can't be lost (x can't be zero) and '0' otherwise
			  					this is called \delta in the Genetics article
			  int save_slots  -->  number of remaining alleles that must have at least one copy realized
			  					   at this point.  Called \kappa in the Genetics article.  
*************/



double
asin_draw_prob(double mean, double var, int x, int n, int no_zero, int save_slots,double schmear_prob)
{
	double prob=0.0;
	double dx,dn;
	double tl=0.0,th;  // theta-low and theta-high
	double rtl, rth; // reflected theta low and reflected theta high (if reflected through
						// the lower bound)
	double htl, hth; // reflected theta low and reflected theta high (if reflected through
						// the high bound)
	double lb;  // the point around which it is reflected
	double hb;
	double contrib;
	int num_step = 12;
	
	if(n == save_slots) {
		if(no_zero == 1) {
			printf("\n\nno_zero is 1 when n == save_slots.  Crashing and burning...\n\n"); 
			printf("\n...exiting program...\n\n");
			exit(1);
		}
		return(1.0);
	}
	
	dx = (double)x;
	dn = (double)n;
	
	if(x>=1)  // if it's a zero, it didn't get drawn from a "middle type" value anyway
			  // and the proper probability will get taken care of by the computation of lb later
		tl = asin(sqrt((dx -.5)/dn));  // "middle-type" theta low and high values that would
	
	if(x < n)
		th = asin(sqrt((dx +.5)/dn));  // yield such an x
	else if(x > n)  {
		printf("\n\nSome bad stuff goin' down!  x > n in asin_draw_prob\n\n");
		printf("\nSorry dude!  But this is a fatal error!\n...Exiting to system...\n");
		exit(1);
	}
	else  // in this case x == n
		th = asin(sqrt((dx)/dn));  // this should evaluate to PI/2.0.
	
	lb = asin(sqrt(.5/(double)n)); 
	if(save_slots > 0) { // if save_slots > 0 the following is well-defined
		hb = asin(sqrt( ((double)n - (double)save_slots + .5 )/(double)n)  );	
	}
	else {   //this corresponds to what goes on in  draw_folded_asin_norm
		hb = asin(sqrt( ((double)n - .5 )/(double)n)  );
	}
	if(no_zero == 1 && save_slots >= 1)  { 
		
		// if x==0, return 0.0; of if x>=n-save_slots  This is a hack right now for the probability checking program
		// in the future though, I should add a little error message here.
		if(x==0 || x > n-save_slots) {
			return(0.0);
		}
		
		// Give it an if so it doesn't compute 1.0 expensively
		if(n == no_zero + save_slots)  {
			return(1.0);
		}
		
		// compute probability of the "middle-type part"
		prob = nd_simp_integ(mean,var,tl,th,num_step);
		
		rtl = 2.0 * lb - th;  // reflected through low bound values
		rth = 2.0 * lb - tl;
		// compute the probability of the lower "reflected and translated part"
		prob += nd_simp_integ(mean,var,rtl,rth,num_step);
		
		contrib = 1.0;
		while(contrib > .0000001) {  // 
			rtl -= (hb - lb);
			rth -= (hb - lb);
			contrib = nd_simp_integ(mean,var,rtl,rth,num_step);
			prob += contrib;
		}
		
		
		htl = 2.0 * hb - th;  // reflected through high bound values
		hth = 2.0 * hb - tl;  
		// compute the probability of the upper "reflected part"
		prob += nd_simp_integ(mean,var,htl,hth,num_step);
		
		contrib = 1.0;
		while(contrib > .0000001) {  // looping to to get the translated parts
			htl += (hb - lb);
			hth += (hb - lb);
			contrib = nd_simp_integ(mean,var,htl,hth,num_step);
			prob += contrib;
		}
	}
	
	if(no_zero == 0 && save_slots >= 1)  { 
		if(x==0) {
			prob = norm_low_tail(mean,var,lb);
		}
		else if(x > n-save_slots)  {  // should also return an error message somewhere 
									  // 'cuz this shouldn't ever happen.  haven't included
									  // that feature yet.
			return(0.0);
		}
		else {
			// compute probability of the "middle-type part"
			prob = nd_simp_integ(mean,var,tl,th,num_step);
			htl = 2.0 *  hb - th;  // reflected through high bound values
			hth = 2.0 *  hb - tl;  
			// compute the probability of the upper "reflected  part"
			prob += nd_simp_integ(mean,var,htl,hth,num_step);
			
			contrib = 1.0;
			while(contrib > .0000001) {  // looping to to get the translated parts
				htl += (hb - lb);
				hth += (hb - lb);
				contrib = nd_simp_integ(mean,var,htl,hth,num_step);
				prob += contrib;
			}
		}	
		
	}	
	
	if(no_zero == 0 && save_slots == 0)  { 
		if(x==0) {
			prob = norm_low_tail(mean,var,lb);
		}
		else if(x==n) {
			prob = norm_up_tail(mean,var,hb);
		}
		else {
			// compute probability of the "middle-type part"
			prob = nd_simp_integ(mean,var,tl,th,num_step);
		}
	}
	
	if(no_zero == 1 && save_slots == 0)  {
		if(x==n) {
			prob = norm_up_tail(mean,var,hb);
		}
		else {
			// compute probability of the "middle-type part"
			prob = nd_simp_integ(mean,var,tl,th,num_step);
			
			rtl = 2.0 * lb - th;  // reflected through low bound values
			rth = 2.0 * lb - tl;
			// compute the probability of the lower "reflected and translated part"
			prob += nd_simp_integ(mean,var,rtl,rth,num_step);
			
			contrib = 1.0;
			while(contrib > .0000001) {  // 
				rtl -= (hb - lb);
				rth -= (hb - lb);
				contrib = nd_simp_integ(mean,var,rtl,rth,num_step);
				prob += contrib;
			}
		}
	}
	
	// this final return statement reflects the probability of the uniform schmear
	return( (1.0-schmear_prob)*prob + schmear_prob*(1.0/(n + 1 - no_zero - save_slots) ) );
}




/*************
*  FUNCTION:  double nd_simp_integ(double mean, double var, double low, double high, int num_steps)

*  PURPOSE:   this function applies simpson's rule to approximate the area under the normal density curve 
			  between two close points 'low' and 'high' by doing num_steps+1 function evaluations.
*
*  INPUT:     double mean -->  the mean of the normal distribution drawn from
			  double var  -->  the variance of the normal distribution drawn from
*  			  double low  -->  the lower endpoint of the interval to be integrated under
			  double high -->  the higher endpoint of the interval to be integrated under
			  int num_steps  -->  the number of pieces to break the area up into. 
*************/





double nd_simp_integ(double mean, double var, double low, double high, int num_steps)
{
	int i;
	double step;
	double prob;
	double len;
	
	len = high - low;
	
	if(num_steps % 2 != 0) {
		printf("\n\nnum_steps must be even, not %d, in nd_simp_integ!\n\n",num_steps);
		printf("\nplease send email to eriq@cqs.washington.edu about this error message\n");
		printf("\n\n...Exiting to system...");
		exit(1);
	}
	step = len/(double)num_steps;
	
	prob = normal_density(mean,var,low);
	
	prob += normal_density(mean,var,high);
	
	for(i=1;i<num_steps;i++)  {
		prob += (2.0 + (double)(i%2)*2.0) * normal_density(mean,var,low + (i * step) );
	}
	
	prob *= (1.0/3.0) * step;
	
	return(prob);
}

/*************
*  FUNCTION:  normal_density(double mean, double var, double x)
*  PURPOSE:   Return the value of the density function for a given
			  normal random variable with mean and var.  
*
*  INPUT:     double mean -->  the mean of the normal distribution 
			  double var  -->  the variance of the normal distribution
*  			  double x  -->  the value whose density we shall return
*************/



double
normal_density(double mean, double var, double x)
{
	return(  (1.0/(sqrt(2.0*3.14159*var) ) ) * 
			exp ( (-pow(x-mean,2))/ (2.0*var) ) );
}


/*************
*  FUNCTION:  double norm_low_tail(double mean, double var, double x)
*  PURPOSE:   NORM(al) LOW(er) TAIL (probability)
*			  Compute the probability that X < x when
			  X is a N(mean,var) random variable.
*
*  INPUT:     double mean -->  the mean of the normal distribution to draw from
			  double var  -->  the variance of the normal distribution to draw from
			  double x  -->  the value of little x
*************/



double 
norm_low_tail(double mean, double var, double x)
{
	double q;
	
	q = (x-mean)/(sqrt(2.0*var));   // q is N(0,1/2).
	
	
	if(q <= 0.0) {
		return(.5 * ErrFunctComp(-q) );
	}
	else {
		return(.5 + .5 *ErrFunct(q) );
	}

}


/*************
*  FUNCTION:  logistic_cdf(double mu, double beta, double x)
*  PURPOSE:   Return the value F(x;mu,beta) of the cdf for a logistic dsn which is 
			  parametrized in terms of its mean and variance.
*
*************/

double
logistic_cdf(double mu, double var, double x)
{
	double beta;
	
	beta = sqrt(3.0*var)/3.1415926535897932385;
	
	return( 1.0 /  
		( 1.0 + exp( -(x-mu)/beta))    );
}


/*************
*  FUNCTION:  double norm_up_tail(double mean, double var, double x)
*  PURPOSE:   NORM(al) LOW(er) TAIL (probability)
*			  Compute the probability that X > x when
			  X is a N(mean,var) random variable.
*
*  INPUT:     double mean -->  the mean of the normal distribution to draw from
			  double var  -->  the variance of the normal distribution to draw from
			  double x  -->  the value of little x
*************/



double 
norm_up_tail(double mean, double var, double x)
{
	double q;
	
	q = (x-mean)/(sqrt(2.0*var));  // q is N(0,1/2).
	
	
	if(q >= 0.0) {
		return(.5 * ErrFunctComp(q) );
	}
	else {
		return(.5 + .5 * ErrFunct(-q) );
	}

}

/*************
*  FUNCTION:  RealizeNew(LatentInfo *inX, GeneInfo *inY, LogProb *inLP, int n_zero)

*  PURPOSE:   For all loci and all time points it generates a realization of the latent variables
			  X from the distribution P^*(X), as described in the Genetics article.  
			  To do this, it transforms allele frequencies by arc-sine square root and does the calculations to realize
			  a set of transformed latent variables, then de-transforms those into the realized
			  x's.  The theta's themselves are drawn using the function draw_folded_asin_norm.  Additionally 
			  P^*(X) is computed by the function asin_draw_prob().  
			  
*
*  INPUT:     LatentInfo *inX  -->  Pointer to the struct that holds the information about the
								    latent allele freqs
			  GeneInfo *inY  -->  Pointer to the struct that holds the information about the observed
			  					     allele counts.
			  LogProb *inLP  --> Pointer to a struct which contains arrays where the result will go
			  
			  int n_zero     --> The effective population size currently assumed.  
	  
	 	THIS VERSION IS SLIGHTLY MODIFIED FROM THE ORIGINAL---I HAVE WRITTEN SOME LINES
		IN THERE SO THAT WHEN THE "HORRIBLE PROBLEM" IS BEING "FIXED" THERE IS STILL EXACTLY
		ONE CALL TO GENNOR FOR EACH TIME STEP.  THIS
		HELPS TO ENSURE THAT WHEN WE TRY COUPLING THINGS, WE GET THE RESULTS THAT
		WE WOULD LIKE TO GET.

		Editor's Note:  "the horrible problem" is the situation that is taken care of by 
		the variable \gamma in the Genetics article.  It is called the "horrible problem"
		because it took a day or two of hand-wringing and worrying before Ellen and Eric
		found and corrected the problem (we hope).  
		
		The implementation in version 1.0 of MCLEEPS had some bugs in this function which were
		discovered while running on a data set by Loic Degen.  They have been fixed in 
		version 1.1.

*************/


void
RealizeNew(LatentInfo *inX, GeneInfo *inY, LogProb *inLP, int n_zero)
{
		
	int *ne;  // for the efffective sizes each generation,  they get subtracted from as alleles are realized
	int save_slots;  // to store each time step, the number of spaces that must be left for "future" alleles
	double temp_log_draw;
	double s[gMAX_GEN];
	double thetameans[gMAX_GEN];
	double thetavars[gMAX_GEN];
	double theta_array[gMAX_GEN];
	int t,j,k,q; 
	int min_index;  // to store the index of the currently minimum pop size.
	int new_min_index;
	double dummy_theta;    // to put the result of gennor when fixing the horrible problem.  This value is not
						   // otherwise used for anything.
	double dummy_mean=.5,dummy_var=.7;  // just some simple values as arguments for the calls to gennor.  I believe that gennor
						   // uses a Box-Muller method, so it should require the same number of calls to the base
						   // r.v. generator, regardless of the mean and variance used.
	
	
	// make a copy of inP and call it theP
	ne = ivector(0,num_gen-1);
	
	// set the notLost field of inX to the appropriate starting values
	inX->RefreshNotLost(inY);
	
	// set the draw probabilities to zero for inLP and set the total draw prob to zero as well
	inLP->SetDrawToZero(inY->numLoci);
	inLP->tot_draw = 0.0;
	
	
	//cycle over loci
	for(j=0;j<num_loc;j++)  {
		
		for(t=0;t<=inX->T;t++) {	// some cycling to initialize the s array
			s[t] = inY->SSZ(t,j);
		}
		
		// each time we start a new locus, we reset the popsize field in theP to be
		// n_zero each generation.
		for(t=0;t<num_gen;t++)  {
			ne[t] = n_zero;
		}
		for(k=0;k<inX->year[0].locus[j].numAlleles-1;k++) { // cycle over alleles...	
															// the last one takes up the slack
			
			// just before we enter the forward cycle, initialize the min_index to zero	
			min_index = 0;
			
															
			//cycle FORWARD over generations to get the array of conditional densities
			for(t=0;t<num_gen;t++) {
				// during this forward cycle, update the value of min_index
				if(ne[t] < ne[min_index]) {
					min_index = t;
				}
							
				if(t==0)  {  // if you only have the "uniform" prior to deal with the density is just
							 // the likelihood of the sample turned "backwards."  
					if(s[t]>0) {		  
						thetameans[t] = asin(sqrt( inY->dCount(t,j,k)/s[t] ) );
						thetavars[t] = 1.0/(4.0 * s[t] ); 
					}
					else {
						thetameans[t] = 0.0;
						thetavars[t] = 1.0;   // this line is klugie.  Think about it and fix it later.
					}
				}
				else {
					// in the absence of a sample, we assume that the mean stays the same
					// note that this might be something we could improve by allowing for the peak
					// of the distribution to shift toward the edges of the space through time
					thetameans[t] = thetameans[t-1];
					
					// in the absence of a sample, the variance increases a little
					thetavars[t] = thetavars[t-1] + 1.0/(4.0 * (double)ne[t] );
					
					if(s[t]>0) { // if there is a sample, then we further modify the mean and variance of
									// these conditional distributions
						thetameans[t] = ( asin(sqrt( inY->dCount(t,j,k)/s[t] ) ) * thetavars[t] + 
											thetameans[t] * (1.0/(4.0*s[t])) ) / 
											(thetavars[t] + (1.0/(4.0*s[t])) ); 
						thetavars[t] = (thetavars[t] * (1.0/(4.0*s[t])) ) / 
											(thetavars[t] + (1.0/(4.0*s[t])) );
					}
				}
			}  // closes the FORWARD t-loop
			

			for(t=num_gen-1;t>=0;t--)  {   // starting the BACKWARD t-loop
			
				// for this time step, determine how many open slots must remain by summing the remaining notLosts
				save_slots = inX->SumRemaining(t,j,k);
				// draw the theta
				

				// ***********  DEAL WITH THE HORRIBLE PROBLEM
				if(t>min_index && ne[min_index] - save_slots <= 0 && inX->year[t].locus[j].notLost[k] == 0 )  {
					// then set a bunch of zeroes up to min_index with probability one
					for(q=t;q>=min_index;q--)  {
						inX->year[q].locus[j].val[k] = 0;					
						
						
						// write an error message if this was a problem
						if(inX->year[q].locus[j].notLost[k] != 0) {
							printf("\n\nStrange Things are Afoot in RealizeNew()");
							printf("\nThe Horrible Problem Fix Just realized X = 0");
							printf("\nWhen NotLost != 0.  Locus j = %d, Allele k = %d, Time t = %d",j,k,q);
							printf("\nExiting to System\n\n");
							exit(1);
						}
						
						// the following two things should only get done for q>min_index, as for
						// t==min_index, they will be done below...
						if(q>min_index)  {
							// And, also, don't forget to update the sample size by subtracting out the number of
							// the current (k) alleles detected in the samples
							if(s[q] > 0 ) {
								s[q] -= (double)inY->sample[q].locus[j].val[k];
							}
							
							// now if we just realized the next to last allele we 
							// put the remaining gene copies in the last allele.  This happens with probability one, of course.
							if(k == inX->year[0].locus[j].numAlleles-2) {
								inX->year[q].locus[j].val[k+1] = ne[q];
							}
						}
						
						// here we must also make a single call to gennor each time through this loop.  The result
						// is not to be used at all, but making the call ensures that we use the same number of 
						// "random bits" for each value of n that we are dealing with.
						dummy_theta = (double)gennor((float)dummy_mean,(float)dummy_var);
					}
					// the probability of that happening was one
					temp_log_draw = 0.0;
					
					// now find the new value for min_index
					if(min_index >= 2) {
						new_min_index = min_index - 2;  // we subtract two, because we are soon going to realize a value for
														// min_index - 1, and what we care about is the smallest current population
														// size BEFORE that!!
						for(q=min_index - 2;q>=0; q--) {
							if (ne[q] <= ne[new_min_index])
								new_min_index = q;
						}
					}
					else {   // otherwise min_index was one or zero, in which case we make the new_min_index 0
						new_min_index = 0;
					}
					// now, we set t to be the time at which we just realized an allele
					// if min_index is zero, then the loop will terminate after executing the 
					// commands after the else{} bit below. 
					t = min_index;						
					min_index = new_min_index;  
					
				}
				
				else { // don't have to fix the horrible problem			
					theta_array[t] = draw_folded_asin_norm(thetameans[t],thetavars[t], ne[t],
						   inX->year[t].locus[j].notLost[k], save_slots, &(inX->year[t].locus[j].val[k] ), 0.0 );  // the 100.0 is just a dummy so
																												// that it fits with the prototype
						   				
					// ******  AND HERE WE MUST UPDATE THE notLost ARRAY to reflect the just-sampled allele
					// remember that the notLost array already has ones where it must, (i.e. where the samples
					// dictate that the notLost array must include a one) so all we need to do is insert a one
					// where a zero exists at time t-1 when a non-zero allele count has just been realized at time t.
					if(t>0 && inX->year[t-1].locus[j].notLost[k] == 0 && inX->year[t].locus[j].val[k] > 0) {
						inX->year[t-1].locus[j].notLost[k] = 1;
					}
					// ***** NOW HEAR THIS!  If we just realized the second to last allele, then we have just determined
					// the realized value for the last allele (we put the remaining available gene copies into the 
					// last allele).  We must accordingly update the notLost array for that last allele. This we do below:
					if(k==inX->year[0].locus[j].numAlleles-2)  {
						if(t>0 && 
							inX->year[t-1].locus[j].notLost[k+1] == 0  && // if the last allele notLost value is 0 at this time
							(ne[t] - inX->year[t].locus[j].val[k]) > 0 )  { // if there was some left over that is going
																			  // to get thrown into the last allele
								inX->year[t-1].locus[j].notLost[k+1] = 1;   // update the notLost array for the last allele
						}
					}
						
					// compute the log of the prob of that x
					temp_log_draw = log(asin_draw_prob(thetameans[t],thetavars[t],inX->year[t].locus[j].val[k],
							ne[t],inX->year[t].locus[j].notLost[k], save_slots, 0.0)  ); // the 100.0 is just a dummy so
																								// that it fits with the prototype				
				
				}		
				// now add it to the draw prob for this locus
				inLP->draw[j] += temp_log_draw;
				
				// add that same quantity to tot_draw while we are at it.
				inLP->tot_draw += temp_log_draw; 
				
				// now update the mean and var for the immediately preceding time step (if not at the beginning already)
				if(t>0){
					// the just-realized x value is like the result of a binomial sample from the previous generation
					// of size equal to the size of the population at time t (psz[t], that is).
					thetameans[t-1] = ( theta_array[t] * thetavars[t-1] + 
										thetameans[t-1] * (1.0/(4.0*(double)ne[t]) ) ) / 
										(thetavars[t-1] + (1.0/(4.0*(double)ne[t]) ) ); 
					thetavars[t-1] = (thetavars[t-1] * (1.0/(4.0*(double)ne[t])) ) / 
										(thetavars[t-1] + (1.0/(4.0*(double)ne[t])) );
					
				}	
				
			
				// and now we update ne[t] by subtracting the value of the latent allele count
				// that we just drew.  Thus ne[t] will be ready for the next allele.
				ne[t] = ne[t] - (inX->year[t].locus[j].val[k]);
				
				// And, also, don't forget to update the sample size by subtracting out the number of
				// the current (k) alleles detected in the samples
				if(s[t] > 0 ) {
					s[t] -= (double)inY->sample[t].locus[j].val[k];
				}
	
					
				// now if we just realized the next to last allele we 
				// put the remaining gene copies in the last allele.  This happens with probability one, of course.
				if(k == inX->year[0].locus[j].numAlleles-2) {
					inX->year[t].locus[j].val[k+1] = ne[t];
				}
			
			} // end of the BACKWARD cycle.
			
		}  // closes the k loop over alleles
		
		
	}  // closes the j loop over loci

	// just before exiting the function, delete the PopInfo struct.

	
	free_ivector(ne,0,num_gen-1);
}



/*************
*  FUNCTION:  uniform(int a, int n)
*  PURPOSE:   Generate a random integer uniformly distributed between a and n,
*			  inclusive.  This uses ranlib's ignuin function which does exactly
*			  the same thing, but I wrote this so it accepts ints instead of longs
*
*  INPUT:     int a  -->  low endpoint of the interval (inclusive)
*			  int n  -->  high endpoint of the interval (inclusive)
*************/


int 
uniform(int a, int n)
{
	return( (int)ignuin( (long)a, (long)n ) );
}


/*************
*  FUNCTION:  
*  PURPOSE:   Write out to a file all the data that were read in
			  so that the user can verify that the settings and data are
			  correct for the run.  
*
*  INPUT:     GeneInfo *inG
			  long seed1
			  long seed2
*************/


void 
echo_data(GeneInfo *inG, int seed1, int seed2)
{
	FILE *EchoFile;
	int t,j,k;
	time_t ProgLaunchedTime;
	
	EchoFile = fopen("aa_SettingsAndDataOut.txt","w");
	
	fprintf(EchoFile,"Data Input File = \"%s\"\n",gINFILE);
	fprintf(EchoFile,"Ne Low:   %d",inG->nLow);
	fprintf(EchoFile,"\nNe High:  %d",inG->nHigh);
	fprintf(EchoFile,"\nNe Incr:  %d",inG->nIncr);
	fprintf(EchoFile,"\nNumber of Reps for each value of Ne:  %d",inG->numReps);
	fprintf(EchoFile,"\nRandom Number Seeds 1 and 2:  %d  %d",seed1,seed2);
	
	fprintf(EchoFile,"\n\nNumber of Samples:  %d\n	Taken at generations:",inG->numSamples);
	for(t=0;t<inG->numSamples;t++)  {
		fprintf(EchoFile,"  %d",inG->gen[t]);
	}
	fprintf(EchoFile,"\nNum Generations between first and last sample:  %d",num_gen-1);
	fprintf(EchoFile,"\nNumber of Loci:  %d", num_loc);
	ProgLaunchedTime = time(NULL);
	fprintf(EchoFile,"\n\nProgram Launched at %s\n",ctime(&ProgLaunchedTime));
	fprintf(EchoFile,"Finishing Time Will Be Printed Below Locus List Upon Completion");
	
	fprintf(EchoFile,"\n\nData for loci as follows:\n");
	
	
	for(j=0;j<num_loc;j++)  {
		fprintf(EchoFile,"\nLocus Number %d",j+1);
		
		for(t=0;t<inG->numSamples;t++)  {
			fprintf(EchoFile,"\n");
			
			for(k=0;k<inG->sample[inG->gen[t]].locus[j].numAlleles;k++) {
				fprintf(EchoFile,"    %d",inG->sample[inG->gen[t]].locus[j].val[k]);
			}
		}
		fprintf(EchoFile,"\n");
	}
	fclose(EchoFile);
	
	
	
}

/*************
*  FUNCTION:  LocByLocPrint
*  PURPOSE:   Write out the likelihood curves and the associated confidence
			  intervals for individual loci.  This thing assumes that before
			  it has gotten called for the first time there is a file named
			  aa_LocByLocLogLOut.txt which has been erased clean, and then had column
			  headers set up on it.
*
*  INPUT:     double *Means
			  double *Vars
			  int inNe  -->  the current value of Ne for which the likelihoods have
			  				 just been computed.
*************/
void 
LocByLocPrint(double *Means, double *Vars, int inNe, double LogNormFactor)
{
	FILE *outfile;
	int j;
	double TotalLike,TotVar,CIup,CIdown;
	
	// open the file in append mode
	outfile = fopen("aa_LocByLocLogLOut.txt","a");

	// first thing on the row, print the current Ne value.  We divide it by two to
	// make it diploid
	fprintf(outfile,"\n%d",inNe/2);
	
	// now, cycle over the loci, printing the appropriate values.  We have to
	// do a few calculations to figure out what the confidence intervals will be
	// so I just cut and paste the corresponding code for that which Ellen wrote
	// in function main() and set the variable values to the appropriate values in 
	// Means and Vars.  But recall that we have to take the darn LogNormFactor out of
	// that, but only one LogNormFactor for each locus.
	for(j=0;j<num_loc;j++)  {
	
		TotalLike = Means[j];
		TotVar = Vars[j];
		
		// print out your estimated confidence intervals for the log-likelihood
		// This is based on Central Limit Theorem Normal approximation
		CIup =log(TotalLike + 1.96*sqrt(TotVar)) + LogNormFactor;
		if (TotalLike > 1.96*sqrt(TotVar)) {
			
			CIdown = log(TotalLike-1.96*sqrt(TotVar)) + LogNormFactor;
			TotalLike = log(TotalLike) + LogNormFactor;
			// ECA has changed these so it prints out as .3f's rather than .10e's (since they _are_ logs!)
			// ECA also has it print out the standard error because we should have that value there.
			fprintf(outfile, "\t%.3f\t%.3f\t%.3f",TotalLike, CIdown, CIup);

		}
		else {
			TotalLike = log(TotalLike) + LogNormFactor;
			// ECA has changed these so it prints out as f's rather than e's (since they _are_ logs!)
			// ECA also has it print out the standard error because we should have that value there.
			fprintf(outfile,"\t%.3f\t-Inf\t%.3f\t",TotalLike, CIup);
		}
	}
	
	// at the very end, close the outfile
	fclose(outfile);
}

// Memory Allocation Functions

// allocate to an array of doubles subscripted from 0 to High.  
// Dummy must be zero.
double *
dvector(int Lo, int High)  
{
	double *temp;
	
	temp = (double *)malloc((size_t)  (High-Lo + 2) * sizeof(double) );
	
	if(temp==NULL)  {
		printf("\n\ndvector failed to allocate to array of length %d",High-Lo+2);
		exit(1);
	}	
		
	return(temp-Lo +1);
}



// free the memory allocated to a double vector
void 
free_dvector(double *Array, int Lo, int High)
{
	int ArrayLength;
	// I don't really use ArrayLength, but have it here so that 
	// Dummy and High are used and so we don't get a compile warning about them
	ArrayLength = High - Lo + 1;
	
	free( (char *) (Array + Lo - 1) );
}

// allocate to an array of ints subscripted from 0 to High.  
// Dummy must be zero.
int *
ivector(int Lo, int High)  
{
	int *temp;
	
	temp = (int *)malloc( (size_t) ((High - Lo + 2) * sizeof(int)) );

	if(temp==NULL)  {
		printf("\n\nivector failed to allocate to array of length %d",High-Lo+2);
		exit(1);
	}	
		
	return (temp-Lo+1);
}

// free the memory allocated to an int vector
void 
free_ivector(int *Array, int Lo, int High)
{
	int ArrayLength;
	// I don't really use ArrayLength, but have it here so that 
	// Dummy and High are used and so we don't get a compile warning about them
	ArrayLength = High - Lo + 1;
	
	free( (char *) (Array+Lo-1));
}


// allocate to a matrix of doubles subscripted from 0 to HiRows and 0 to HiCols
double **
dmatrix(int LoRows, int HiRows, int LoCols, int HiCols)
{
	int i,NumRows,NumCols;
	double **temp;
	
	NumRows = HiRows - LoRows + 1;
	NumCols = HiCols - LoCols + 1;
		
	temp = (double **)malloc( (size_t) ((NumRows+1) * sizeof(double *) ) );
	
	if(temp==NULL)  {
		printf("\n\ndmatrix failed to allocate to array of pointers of length %d",HiRows+1);
		exit(1);
	}
	
	temp += 1;
	temp -= LoRows;
	
	temp[LoRows] = (double *)malloc( (size_t)( (NumRows*NumCols+1)*sizeof(double)));
	
	if(temp[LoRows]==NULL)  {
		printf("\n\ndmatrix failed to allocate to array of doubles of length %d",HiRows+1);
		exit(1);
	}
	temp[LoRows] += 1;
	temp[LoRows] -= LoCols;
	
	for(i=LoRows+1;i<=HiRows;i++)  {
		temp[i] = temp[i-1] + NumCols;
	}
	
	return(temp);
}

void
free_dmatrix(double **Matrix, int LoRows, int HiRows, int LoCols, int HiCols)
{
	int silly;
	
	silly = HiCols - HiRows;
	
	free( (char *) (Matrix[LoRows] + LoCols - 1));
	free( (char *) (Matrix + LoRows -1));
}	


// allocate to a matrix of ints subscripted from 0 to HiRows and 0 to HiCols
int **
imatrix(int LoRows, int HiRows, int LoCols, int HiCols)
{
	int i,NumRows,NumCols;
	int **temp;
	
	NumRows = HiRows - LoRows + 1;
	NumCols = HiCols - LoCols + 1;
		
	temp = (int **)malloc( (size_t) ((NumRows+1) * sizeof(int *) ) );
	
	if(temp==NULL)  {
		printf("\n\nimatrix failed to allocate to array of pointers of length %d",HiRows+1);
		exit(1);
	}
	
	temp += 1;
	temp -= LoRows;
	
	temp[LoRows] = (int *)malloc( (size_t)( (NumRows*NumCols+1)*sizeof(int)));
	
	if(temp[LoRows]==NULL)  {
		printf("\n\nimatrix failed to allocate to array of doubles of length %d",HiRows+1);
		exit(1);
	}
	temp[LoRows] += 1;
	temp[LoRows] -= LoCols;
	
	for(i=LoRows+1;i<=HiRows;i++)  {
		temp[i] = temp[i-1] + NumCols;
	}
	
	return(temp);
}

void
free_imatrix(int **Matrix, int LoRows, int HiRows, int LoCols, int HiCols)
{
	int silly;
	
	silly = HiCols - HiRows;
	
	free( (char *) (Matrix[LoRows] + LoCols - 1));
	free( (char *) (Matrix + LoRows -1));
}	




/*************
*  FUNCTION:  double ErrFunct
*  PURPOSE:   Compute the ErrorFunction
			  
			
*  INPUT:     double x
   
   	Requires the function GammaP.  Based on Numerical Recipes in C, page 220.
   	
   AUTHOR:  Eric C. Anderson
   DATE:  18 AUGUST 2000
*************/
double 
ErrFunct(double x) 
{
	if(x < 0.0) {
		return( -GammaP(.5, x*x));
	}
	else {
		return( GammaP(.5, x*x));
	}
}

/*************
*  FUNCTION:  double ErrFunctComp
*  PURPOSE:   Compute the ErrorFunction
			  
			
*  INPUT:     double x
   
   	Requires the function GammaP and GammaQ  Based on Numerical Recipes in C, page 220.
   	
   AUTHOR:  Eric C. Anderson
   DATE:  18 AUGUST 2000
*************/
double 
ErrFunctComp(double x) 
{
	if(x < 0.0) {
		return( 1.0 + GammaP(.5, x*x));
	}
	else {
		return( GammaQ(.5, x*x));
	}
}

/*************
*  FUNCTION:  double GammaP
*  PURPOSE:   Compute the incomplete Gamma function.  P(a,x)
			  
			
*  INPUT:     double a;
			  double x;
   
   	Requires the functions IncGammaCF and IncGammaSer
   	
   AUTHOR:  Eric C. Anderson
   DATE:  18 AUGUST 2000
*************/
double 
GammaP(double a, double x) 
{
	if(x<0.0 || a <= 0.0)  {
		printf("\n\nx<0.0 or a<=0.0 in GammaP\n\nExiting to system...");
		exit(1);
	}
	if(x <= a + 1.0) {
		return(IncGammaSer(a,x));
	}
	else {
		return(1.0 - IncGammaCF(a,x));
	}
}


/*************
*  FUNCTION:  double GammaQ
*  PURPOSE:   Compute the incomplete Gamma function.  Q(a,x).
			  This is the part integrated from x to infinity.
			  
			
*  INPUT:     double a;
			  double x;
   
   	Requires the functions IncGammaCF and IncGammaSer
   	
   AUTHOR:  Eric C. Anderson
   DATE:  18 AUGUST 2000
*************/
double 
GammaQ(double a, double x) 
{
	if(x<0.0 || a <= 0.0)  {
		printf("\n\nx<0.0 or a<=0.0 in GammaP\n\nExiting to system...");
		exit(1);
	}
	if(x <= a + 1.0) {
		return(1.0 - IncGammaSer(a,x));
	}
	else {
		return(IncGammaCF(a,x));
	}
}

/*************
*  FUNCTION:  double IncGammaSer
*  PURPOSE:   Compute the incomplete Gamma function by its series representation.  
			  Note that this computes the part from 0 to x (called P(a,x) )
			  
			  This is the preferred method when x < (a+1)
			
*  INPUT:     double x

	This function is based on the series representation described
	in "Numerical Recipes in C" (Cambridge University Press), 
   
   	Requires the function GammaLog
   	
   AUTHOR:  Eric C. Anderson
   DATE:  18 AUGUST 2000
*************/
#define MAXIT 100
#define PRECIS 1.0e-7

double 
IncGammaSer(double a, double x) 
{
	double old_funct, new_funct;  // this is what we ultimately want to return;
	double term;  // the next term to be added to the series
	double i;
	double numer;  // the normalizing constant---Gamma(a).  Also turns out to be the numerator of the terms that
				   // get summed.
	double denom;  // the denominator of the terms that get summed
	
	// set value of the log of the normalizing constant
	numer = exp(LogGamma(a));
	denom = a * numer;
	
	term = numer/denom;
	old_funct = term;
	
	for(i=1;i<MAXIT;i++)  {
		term *= 1.0/(a + i) * x;  // the terms may be computed recursively
		new_funct = old_funct + term;
		
		// check for convergence
		if( fabs(new_funct - old_funct) < PRECIS)  {
			break;
		}
		else {
			old_funct = new_funct;
		}
	}
	
	if(i >= MAXIT - .001) {
		// done here we warn if it failed to converge. 
		printf("\n\nFailed to converge in IncGammaSer\n\n");
		exit(1);
	}
	
	// otherwise, we return the result
	// first we have to add the coefficients and normalize it:
	new_funct *= exp(-x) * pow(x,a) * 1.0/numer;
	return(new_funct);
}
#undef MAXIT
#undef PRECIS

/*************
*  FUNCTION:  double IncGammaCF
*  PURPOSE:   Compute the incomplete Gamma function by its continued
			  fraction representation.  Note that this computes the 
			  part from x to infinity (called Q(a,x) )
			  
			  This is the preferred method for x > (a + 1)
			
*  INPUT:     double x

	This function is based on the continued fraction representation described
	in "Numerical Recipes in C" (Cambridge University Press), and evaluated using
	Lentz's method for continued fractions.  Basically following the pseudocode on
	page 171.
   
   	Requires the function GammaLog
   	
   AUTHOR:  Eric C. Anderson
   DATE:  18 AUGUST 2000
*************/
#define SMIDGEN 1.0e-30
#define MAXIT 100
#define PRECIS 2.0e-7

double 
IncGammaCF(double a, double x) 
{
	double funct;  // this is what we ultimately want to return;
	double A,B,C,D;
	double Delta;
	double j;
	double log_normo;  // the log of the normalizing constant---Gamma(a)
	
	// set value of the log of the normalizing constant
	log_normo = LogGamma(a);
	
	// here for the zero subscript part:
	B = 0.0;  // b_0 is really zero, so we just make it tiny
	funct = SMIDGEN;
	C = funct;
	D = 0.0;
	
	// then for the 1 subscript part things are sort of different still
	A = 1.0;
	B = x + 1.0 - a;
	D = B + A * D;
	if(fabs(D) < SMIDGEN) D = SMIDGEN;
	C = B +  A/C;
	if(fabs(C) < SMIDGEN) C = SMIDGEN;
	D = 1.0/D;
	Delta = C * D;
	funct = Delta * funct;
	// don't even bother checking for convergence after this first "iteration"
	
	
	// now we iterate through the 2,3... and so forth subscript parts until converged
	// so each loop corresponds to the j+1-th subscript of the continued fraction
	for(j=1.0;j<MAXIT;j++)  {
		// now, we define the new values for A and B on the j-th level of the continued fraction
		A = -j * (j - a);
		B = x + 1.0 + (2 * j) - a;	
		
		D = B + A * D;
		if(fabs(D) < SMIDGEN) D = SMIDGEN;
		C = B + A/C;
		if(fabs(C) < SMIDGEN) C = SMIDGEN;
		D = 1.0/D;
		Delta = C * D;
		funct = Delta * funct;

		// here we check for convergence.  If it has, we return the appropriate result
		if(fabs(Delta - 1.0) < PRECIS) {
			// add the coefficients and the normalizing constant
			funct *= exp(-x - log_normo) * pow(x,a);
			return(funct);
		}
	}
	
	// done here we warn if it failed to converge.  Should never get here
	printf("\n\nFailed to converge in IncGammaCF\n\n");
	exit(1);
	
	return(-55.55);  // just put this in so that it doesn't give a compile warning
					 // about having no return value.

}
#undef SMIDGEN
#undef MAXIT
#undef PRECIS

/*************
*  FUNCTION:  double LogGamma
*  PURPOSE:   Compute log of the Gamma Function
			
*  INPUT:     double x

	This function is based on the six term series of Lanczos as described
	in "Numerical Recipes in C" (Cambridge University Press).  I use the 
	choice of gamma = 5 and N = 6.  
	
*************/
double 
LogGamma(double x) 
{
	// declare the coefficients in the series as a static double array
	static double Coeff[6] = {76.18009172947146,-86.50532032941677,
		24.01409824083091,-1.231739572450155,
		0.1208650973866179e-2,-0.5395239384953e-5};
	double Prelim;  // for the part before the series
	double Series;
	int i;
	double denom;	
	
	// We start off by computing Gamma(x+1) by the series:
	
	// compute the preliminary part
	Prelim = (x+.5) * log(x+5.5) - (x+5.5);

	// add the log of sqrt(2\pi) to that:
	Prelim += 0.91893853320467274;
	
	// now compute the series.  Start with the constant term c_0
	Series = 1.000000000190015;

	denom = x;
	for(i=0;i<6;i++)  {
		Series += Coeff[i]/(++denom);
	}
	
	// now we just have to return the right thing.  Since we just computed
	// Recall Gamma(x+1) = x * Gamma(x), so we have to subtract log(x) from this.
	// We can do this with only one call to log by dividing Series by x.
	return(Prelim + log(Series/x));


}

/*************
*  FUNCTION:  double LogFactorial
*  PURPOSE:   Compute the log of x!
			  
			
*  INPUT:     double x
   
   	Requires the function LogGamma
   	
   	Based on the implementation in Numerical Recipes in C.
   	
   AUTHOR:  Eric C. Anderson
   DATE:  18 AUGUST 2000
*************/
double 
LogFactorial(int x) 
{
	static double PreStored[201];  // we will precompute and store value for up to 200
	int i;
	
	if(x<0) {
		printf("\n\nNegative argument in call of LogFactorial\n\nExiting...");
		exit(1);
	}
	if(x<=1) {
		return(0.0);
	}
	if(x <= 200)  {
		if(!PreStored[x])  {  // if PreStored[x] is zero, then we must fill in the table
			for(i=2;i<201;i++)  {
				PreStored[i] = LogGamma((double)i + 1.0);
			}
		}
		return(PreStored[x]);
	}
	else    // otherwise we have to compute it.
		return(LogGamma((double)x + 1.0));
	
}


#ifdef MAC_CODE_WARRIOR  // this one doesn't work on most Unix systems.
/*************
*  FUNCTION:  void RestoreTheScreen
*  PURPOSE:   For the Mac version now, at least, this erases the current
			  contents of the SIOUX window and then gives information about
			  which data file is being used and how many reps it has completed
			  and how many it has to go.  Also, it computes how long it has
			  been running and how long it has to go until completion based on 
			  how long it has taken it to get to where it is at the moment. 
			  
   AUTHOR:  Eric C. Anderson
   DATE:  30 AUGUST 2000
*************/

void RestoreTheScreen(GeneInfo *inG, int seed1, int seed2, int n_zero,
			int CurrentRep, int NumNe, int num_reps,int IsStopped)
{
	int t;
	int Hours,Minutes,Seconds;
	time_t CurrentTime;
	int ElapsedSeconds;
	int TotalReps = NumNe * num_reps;
	double SecondsPerRep;
	int RemainingReps;
	int RepsUpTillNow;
	int RemainingSeconds;
	
	CurrentTime = time(NULL);
	ElapsedSeconds = (int)difftime(CurrentTime,gStartTime);
	Hours = ElapsedSeconds / 3600;
	Minutes = (ElapsedSeconds % 3600) / 60;
	Seconds = ElapsedSeconds % 60;
	
	// compute the number of reps that have been completed
	RepsUpTillNow = ((n_zero/2 - inG->nLow) / inG->nIncr) * num_reps + CurrentRep;
	SecondsPerRep = (double)ElapsedSeconds / (double)RepsUpTillNow;
	
	RemainingReps = TotalReps - RepsUpTillNow;
	RemainingSeconds = (int)((double)RemainingReps * (double)SecondsPerRep);
	
	
	clrscr();
	if(!IsStopped)
		printf("Currently Executing Program MCLEEPS");
	else {
		printf("*******  Program MCLEEPS is Currently Suspended  *******");
	}
	printf("\nUsing Data File: \"%s\"\n",gINFILE);
			
	printf("Ne Low:   %d",inG->nLow);
	printf("\nNe High:  %d",inG->nHigh);
	printf("\nNe Incr:  %d",inG->nIncr);
	printf("\nNumber of Reps for each value of Ne:  %d",inG->numReps);
	printf("\nRandom Number Seeds 1 and 2:  %d  %d",seed1,seed2);
	
	printf("\nNumber of Samples:  %d\nTaken at generations:",inG->numSamples);
	for(t=0;t<inG->numSamples;t++)  {
		printf("   %d",inG->gen[t]);
	}
	printf("\nNum Generations between first and last sample:  %d",num_gen-1);
	printf("\nNumber of Loci:  %d", num_loc);
	printf("\n-------------");
	printf("\nCURRENTLY ON N_e = %d",n_zero/2);
	printf("\nTime since execution began:");
	if(Hours==1) 
		printf("  %d Hour",Hours);
	if(Hours > 1)
		printf("  %d Hours",Hours);
	if(Minutes > 1 || (Minutes==0 && Hours > 0)) 
		printf("  %d Minutes",Minutes);
	if(Minutes == 1)
		printf("  %d Minute",Minutes);
	printf("  %d Seconds\n",Seconds);
	
	if( ((n_zero/2) > inG->nLow) || CurrentRep > 0 ) {  // only if some reps have
														// been completed.
		// compute Hours Seconds and Minutes for the remaining time
		Hours = RemainingSeconds / 3600;
		Minutes = (RemainingSeconds % 3600) / 60;
		Seconds = RemainingSeconds % 60;
		printf("Approx. time to completion:");
		if(Hours==1) 
			printf("  %d Hour",Hours);
		if(Hours > 1)
			printf("  %d Hours",Hours);
		if(Minutes > 1 || (Minutes==0 && Hours > 0)) 
			printf("  %d Minutes",Minutes);
		if(Minutes == 1)
			printf("  %d Minute",Minutes);
		printf("  %d Seconds",Seconds);
	}
	printf("\nCURRENTLY ON REP %d",CurrentRep);
	if(!IsStopped) 
		printf(" + number of dots below\n");
}
#endif








