// Here is a toggle.  Should be #defined only if compiling on Mac using CodeWarrior
//#define MAC_CODE_WARRIOR

// defs.h

#ifndef DEFS_H
#define DEFS_H


// SOME MAX ALLOWABLE SIZES FOR STATICALLY DECLARED ARRAYS
// MAKE THEM QUITE LARGE
#define gMAX_ALLELES 50
#define gMAX_SAMPLES 40
#define gMAX_GEN 300
#define gMAX_LOC 300


#define PI 3.141592654

#define FALSE 0
#define TRUE 1

class Locus {
  public:
  Locus() {}
 // Locus(int inNumAlleles);
  void init(int inNumAlleles);
  ~Locus();
  Locus &operator=(Locus &inLocus);
  int numAlleles;
	int sz; // sample size at this locus at this timepoint
	int *val; // array, number of copies of each allele in sample
	int *notLost;
};
typedef Locus Locus;

struct Sample {
  Locus *locus;
};
typedef Sample Sample;

class PopInfo {
	public:
		PopInfo(int inT, int inN);
		~PopInfo();
		void PutConstPsz(int inN);
		PopInfo &operator=(PopInfo &inPopInfo);
		int T;
		int *psz;
}; 
typedef PopInfo PopInfo;

class GeneInfo {
  public:
		void PrintAll(void);
		GeneInfo();
		~GeneInfo();
		double dCount(int t, int j, int k) {
			return( (double)sample[t].locus[j].val[k]);
		}
		double SSZ(int t, int j) {
			return((double)sample[t].locus[j].sz);
		}
		void SortAlleles();
		int *YArray(int t, int j) {
		  return sample[t].locus[j].val;
		}
		int numReps;
		int nHigh;
		int nLow;
		int nIncr;
		int numSamples; // how many times samples were taken
		int numLoci;
		int T;
		int *gen; // generation at which samples are taken
		Sample *sample;
};
typedef class GeneInfo GeneInfo;

class LatentInfo  {  // an ECA addition
	public:
		LatentInfo(GeneInfo *G);
		~LatentInfo();
		void FreqArray(int t, int j, double *p, int n_zero); 
		double Freq(int t, int j, int k, int n_zero)  {
				return( (double)year[t].locus[j].val[k]/(double)n_zero );
		}
		int *XArray(int t, int j) {
		  return year[t].locus[j].val;
		}
		
		int SumRemaining(int t, int j, int k); 
		void RefreshNotLost(GeneInfo *inG);
		int T;
		int numLoci;
		Sample *year;
};
typedef class LatentInfo LatentInfo;

class LogProb {  // another ECA addition
    public:
	    LogProb(int inNumLoci);
	    
	   // LogProb(GeneInfo *inG);
	    ~LogProb();
	    void SetDrawToZero(int inNumLoci); 
		double tot_draw;
		double tot_joint;
		double *draw;
		double *joint;
};
typedef LogProb LogProb;


class ListEntry {
public:
  ListEntry();
  struct ListEntry *nxt;
        double val;
};
typedef class ListEntry ListEntry;

class SpecialListEntry {
public:
  SpecialListEntry();
  struct SpecialListEntry *nxt;
        int val;
		int orig; // the original index for val
};
typedef class SpecialListEntry SpecialListEntry;

class StoredInfo {
	public:
		StoredInfo(GeneInfo *inG);
		double *Prev_Av;  // array of previous averages for each locus
		LatentInfo *LI;
		LogProb *LP; 
		double *theLogRatios;
};

#endif // DEFS_H



// ORIGINALLY Util.h
// functions.h
#ifndef FCN_H
#define FCN_H

#include <stdlib.h>
#include <stdio.h>
//#include <stream.h>
#include <fstream.h>
#include <iostream.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

// include this if Compiling on the Mac for
// checking during the loop to see if the user wants to stop the loop for a moment 
#ifdef MAC_CODE_WARRIOR
#include <console.h>
#endif


#define max(A, B) ((A) > (B) ? (A) : (B))
#define min(A, B) ((A) < (B) ? (A) : (B))

// some other functions
long AChooseB(long A, long B);
void Sort(double *inArray, long inSize);
long FindMax(double *theList, long theLen);
int *SpecialSort(int *inArray, int inSize);
#endif 


/* buffer.h */

#ifndef BUFFER_H
#define BUFFER_H
#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>

// constants for Ne_Classes.C
const int E_FALSE = 0;
const int E_TRUE = 1;

// a global variable for the file name
char gINFILE[300];   // this stores the name for the infile

class CBuffer {
  public:
	CBuffer(void);
	CBuffer(char *inFileName);
  ~CBuffer();
  ifstream mInFile;    
  char *mBuffer;
	long mBufPos; // current position in the buffer
	long mBufSize; // number of bytes in the buffer
	int mIsEndInBuffer; // is the End Of File (EOF) currently in the buffer
	int mIsAtEnd; // are we there, at the very end of the file, now  
	int IsAtEnd(void) { return mIsAtEnd; }
	void OpenBuffer(char *inFileName);
	void CloseBuffer(void);
	void ResetBuffer(void) { mBufPos = 0; mIsAtEnd = FALSE;}
	char GetNextByte(); // updates buf_pos, returns next byte of data
	void FillBuffer();
	void SkipSpace();
	void SkipOneCol();
	void SkipCols(long);
	int CountCols(void);
  void MoveToNextRow();
	void SkipRows(long);
  double GetFloat();
  int GetInt();
  char *GetString();
  unsigned char *GetPascalString();  
  float InspectFloatPast(long);
  char InspectByte(long);
  long BytesToNextCol(long);
  long BytesToRow(long);
};
typedef class CBuffer CBuffer;
#endif // BUFFER_H


/* buf_con.h */

#ifndef BUF_CON_H // prevent multiple inclusions
#define BUF_CON_H

/* extern constants from buffer.C */
const int MAX_NAMELEN  = 100;
const int MAX_BUF =  6000;
#endif // BUF_CON_H

/* This is the header file that contains prototypes for all the
functions used in the EFFPOP program.  Each file containing
functions should have a 
							#include "effpop.h"
statement. */

/* Conditional preprocessing.  In all files except the one in which
the function main()  resides, the preprocessor replaces "GLOB" with "extern".
The file containing main() should start with the line:

							#define UN_EXTERN
*/							

#ifndef UN_EXTERN
	#define GLOB extern
#else
	#define GLOB 
#endif

#define ANSI // so as to get the ANSI versions of numerical recipes 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "ranlib.h"


/* some global variables  */
GLOB int num_gen;
GLOB int num_loc;
GLOB int Global_SortByThisLocus;
GLOB time_t gStartTime;

GLOB int uniform(int a, int n);
GLOB long ignuin(long a, long n);
GLOB float ranf(void);
GLOB int porn(void);
GLOB int duwox(int a, int n, int x);
GLOB double prior(int x);



// Memory Allocation Prototypes
GLOB double *dvector(int Dummy, int High);
GLOB void free_dvector(double *Array, int Dummy, int High);
GLOB int *ivector(int Dummy, int High);
GLOB void free_ivector(int *Array, int Dummy, int High);
GLOB double **dmatrix(int Dummy1, int HiRows, int Dummy2, int HiCols);
GLOB void free_dmatrix(double **Matrix, int Dummy1, int HiRows, int Dummy2, int HiCols);
GLOB int **imatrix(int Dummy1, int HiRows, int Dummy2, int HiCols);
GLOB void free_imatrix(int **Matrix, int Dummy1, int HiRows, int Dummy2, int HiCols);

// prototypes for some of the numerical calculations
GLOB double LogGamma(double x);
GLOB double IncGammaCF(double a, double x);
GLOB double IncGammaSer(double a, double x);
GLOB double GammaP(double a, double x);
GLOB double GammaQ(double a, double x) ;
GLOB double ErrFunct(double x);
GLOB double ErrFunctComp(double x);
GLOB double LogFactorial(int x);

// prototypes for some other important functions
GLOB double norm_up_tail(double mean, double var, double x);
GLOB double norm_low_tail(double mean, double var, double x);
GLOB double normal_density(double mean, double var, double x);
GLOB double asin_draw_prob(double mean, double var, int x, int n, int no_zero, int save_slots, double schmear_prob);
GLOB double draw_folded_asin_norm(double mean,double var, int n, int no_zero, int save_slots, int *x, double schmear_prob);
GLOB double nd_simp_integ(double mean, double var, double low, double high, int num_steps);
GLOB void echo_data(GeneInfo *inG, int seed1, int seed2);
GLOB void LocByLocPrint(double *Means, double *Vars, int inNe, double LogNormFactor);
GLOB double logistic_cdf(double mu, double var, double x);
GLOB void RestoreTheScreen(GeneInfo *inG, int seed1, int seed2, int n_zero,
			int CurrentRep, int NumNe, int num_reps,int IsStopped);

// working with ellen things
GLOB void RealizeNew(LatentInfo *inX, GeneInfo *inY, LogProb *inLP, int n_zero);
GLOB double log_multi_prob(int n, double *p, int *x, int inNum);
GLOB void log_joint_multi_prob(LatentInfo *inX, GeneInfo *inY, LogProb *inLP, int n_zero);

